@@ -54,6 +54,7 @@ typedef enum {
 
 typedef struct _GstBin GstBin;
 typedef struct _GstBinClass GstBinClass;
+typedef struct _GstBinPrivate GstBinPrivate;
 
 /**
  * GST_BIN_NUMCHILDREN:
@@ -85,7 +86,7 @@ typedef struct _GstBinClass GstBinClass;
  * @child_bus: internal bus for handling child messages
  * @messages: queued and cached messages
  * @polling: the bin is currently calculating its state
- * @state_dirty: the bin needs to recalculate its state
+ * @state_dirty: the bin needs to recalculate its state (deprecated)
  * @clock_dirty: the bin needs to select a new clock
  * @provided_clock: the last clock selected
  * @clock_provider: the element that provided @provided_clock
@@ -114,7 +115,9 @@ struct _GstBin {
   GstElement    *clock_provider;
 
   /*< private >*/
-  gpointer _gst_reserved[GST_PADDING];
+  GstBinPrivate *priv;
+
+  gpointer _gst_reserved[GST_PADDING - 1];
 };
 
 /**
@@ -161,7 +164,7 @@ gboolean	gst_bin_remove			(GstBin *bin, GstElement *element);
 /* retrieve a single child */
 GstElement*	gst_bin_get_by_name		 (GstBin *bin, const gchar *name);
 GstElement*	gst_bin_get_by_name_recurse_up	 (GstBin *bin, const gchar *name);
-GstElement*	gst_bin_get_by_interface	 (GstBin *bin, GType interface);
+GstElement*	gst_bin_get_by_interface	 (GstBin *bin, GType iface);
 
 /* retrieve multiple children */
 GstIterator*    gst_bin_iterate_elements	 (GstBin *bin);
@@ -170,7 +173,7 @@ GstIterator*    gst_bin_iterate_recurse		 (GstBin *bin);
 
 GstIterator*	gst_bin_iterate_sinks		 (GstBin *bin);
 GstIterator*	gst_bin_iterate_sources		 (GstBin *bin);
-GstIterator*	gst_bin_iterate_all_by_interface (GstBin *bin, GType interface);
+GstIterator*	gst_bin_iterate_all_by_interface (GstBin *bin, GType iface);
 
 G_END_DECLS
 
