@@ -895,7 +895,7 @@ gst_caps_set_simple_valist (GstCaps * caps, char *field, va_list varargs)
   GstStructure *structure;
 
   g_return_if_fail (GST_IS_CAPS (caps));
-  g_return_if_fail (caps->structs->len != 1);
+  g_return_if_fail (caps->structs->len == 1);
   g_return_if_fail (IS_WRITABLE (caps));
 
   structure = gst_caps_get_structure (caps, 0);
@@ -1214,9 +1214,15 @@ gst_caps_intersect (const GstCaps * caps1, const GstCaps * caps2)
   g_return_val_if_fail (GST_IS_CAPS (caps1), NULL);
   g_return_val_if_fail (GST_IS_CAPS (caps2), NULL);
 
-  if (gst_caps_is_empty (caps1) || gst_caps_is_empty (caps2)) {
+  /* caps are exactly the same pointers, just copy one caps */
+  if (caps1 == caps2)
+    return gst_caps_copy (caps1);
+
+  /* empty caps on either side, return empty */
+  if (gst_caps_is_empty (caps1) || gst_caps_is_empty (caps2))
     return gst_caps_new_empty ();
-  }
+
+  /* one of the caps is any, just copy the other caps */
   if (gst_caps_is_any (caps1))
     return gst_caps_copy (caps2);
   if (gst_caps_is_any (caps2))
@@ -1791,14 +1797,20 @@ gst_caps_to_string (const GstCaps * caps)
     GstStructure *structure;
     char *sstr;
 
-    if (i > 0)
-      g_string_append (s, "; ");
+    if (i > 0) {
+      /* ';' is now added by gst_structure_to_string */
+      g_string_append (s, " ");
+    }
 
     structure = gst_caps_get_structure (caps, i);
     sstr = gst_structure_to_string (structure);
     g_string_append (s, sstr);
     g_free (sstr);
   }
+  if (s->len && s->str[s->len - 1] == ';') {
+    /* remove latest ';' */
+    s->str[--s->len] = '\0';
+  }
 
   return g_string_free (s, FALSE);
 }
@@ -1824,22 +1836,20 @@ gst_caps_from_string_inplace (GstCaps * caps, const gchar * string)
   }
   gst_caps_append_structure (caps, structure);
 
-  while (*s == ';') {
-    s++;
+  do {
+
     while (g_ascii_isspace (*s))
       s++;
+    if (*s == '\0') {
+      break;
+    }
     structure = gst_structure_from_string (s, &s);
     if (structure == NULL) {
       return FALSE;
     }
     gst_caps_append_structure (caps, structure);
-    while (g_ascii_isspace (*s))
-      s++;
-  }
 
-  if (*s != 0) {
-    return FALSE;
-  }
+  } while (TRUE);
 
   return TRUE;
 }
