@@ -23,7 +23,7 @@
 
 /**
  * SECTION:gstinfo
- * @short_description: Debugging and logging facillities
+ * @short_description: Debugging and logging facilities
  * @see_also: #GstConfig, #Gst for command line parameters
  * and environment variables that affect the debugging output.
  *
@@ -51,7 +51,7 @@
  * the variables and set the default category.
  * <informalexample>
  * <programlisting>
- * GST_DEBUG_CATEGORY (my_category);            // define category
+ * GST_DEBUG_CATEGORY_STATIC (my_category);     // define category (statically)
  * &hash;define GST_CAT_DEFAULT my_category     // set as default
  * </programlisting>
  * </informalexample>
@@ -77,7 +77,7 @@
  * subsystem was compiled out, #GST_DISABLE_GST_DEBUG is defined in
  * &lt;gst/gst.h&gt;,
  * so you can check that before doing your trick.
- * Disabling the debugging subsystem will give you a slight (read: unnoticable)
+ * Disabling the debugging subsystem will give you a slight (read: unnoticeable)
  * speed increase and will reduce the size of your compiled code. The GStreamer
  * library itself becomes around 10% smaller.
  *
@@ -104,8 +104,10 @@
 #  include <process.h>          /* getpid on win32 */
 #endif
 #include <string.h>             /* G_VA_COPY */
+
 #include "gst_private.h"
 #include "gstutils.h"
+#include "gstsegment.h"
 #ifdef HAVE_VALGRIND
 #  include <valgrind/valgrind.h>
 #endif
@@ -113,8 +115,12 @@
 
 /* underscore is to prevent conflict with GST_CAT_DEBUG define */
 GST_DEBUG_CATEGORY_STATIC (_GST_CAT_DEBUG);
-/* time of initialization, so we get useful debugging output times */
-static GstClockTime start_time;
+
+/* time of initialization, so we get useful debugging output times
+ * FIXME: we use this in gstdebugutils.c, what about a function + macro to
+ * get the running time: GST_DEBUG_RUNNING_TIME
+ */
+GstClockTime _priv_gst_info_start_time;
 
 #if 0
 #if defined __sgi__
@@ -149,7 +155,9 @@ static void gst_debug_reset_threshold (gpointer category, gpointer unused);
 static void gst_debug_reset_all_thresholds (void);
 
 #ifdef HAVE_PRINTF_EXTENSION
-static int _gst_info_printf_extension (FILE * stream,
+static int _gst_info_printf_extension_ptr (FILE * stream,
+    const struct printf_info *info, const void *const *args);
+static int _gst_info_printf_extension_segment (FILE * stream,
     const struct printf_info *info, const void *const *args);
 static int _gst_info_printf_extension_arginfo (const struct printf_info *info,
     size_t n, int *argtypes);
@@ -192,8 +200,7 @@ static gint __use_color;
 /* disabled by default, as soon as some threshold is set > NONE,
  * it becomes enabled. */
 gboolean __gst_debug_enabled = FALSE;
-static gboolean __gst_debug_min = GST_LEVEL_NONE;
-
+GstDebugLevel __gst_debug_min = GST_LEVEL_NONE;
 
 GstDebugCategory *GST_CAT_DEFAULT = NULL;
 
@@ -277,11 +284,13 @@ _gst_debug_init (void)
 
   /* get time we started for debugging messages */
   g_get_current_time (&current);
-  start_time = GST_TIMEVAL_TO_TIME (current);
+  _priv_gst_info_start_time = GST_TIMEVAL_TO_TIME (current);
 
 #ifdef HAVE_PRINTF_EXTENSION
-  register_printf_function (GST_PTR_FORMAT[0], _gst_info_printf_extension,
+  register_printf_function (GST_PTR_FORMAT[0], _gst_info_printf_extension_ptr,
       _gst_info_printf_extension_arginfo);
+  register_printf_function (GST_SEGMENT_FORMAT[0],
+      _gst_info_printf_extension_segment, _gst_info_printf_extension_arginfo);
 #endif
 
   /* do NOT use a single debug function before this line has been run */
@@ -487,10 +496,72 @@ gst_debug_print_object (gpointer ptr)
   if (G_IS_OBJECT (object)) {
     return g_strdup_printf ("<%s@%p>", G_OBJECT_TYPE_NAME (object), object);
   }
+  if (GST_IS_MESSAGE (object)) {
+    GstMessage *msg = GST_MESSAGE_CAST (object);
+    gchar *s, *ret;
+
+    if (msg->structure) {
+      s = gst_structure_to_string (msg->structure);
+    } else {
+      s = g_strdup ("(NULL)");
+    }
+
+    ret = g_strdup_printf ("%s message from element '%s': %s",
+        GST_MESSAGE_TYPE_NAME (msg), (msg->src != NULL) ?
+        GST_ELEMENT_NAME (msg->src) : "(NULL)", s);
+    g_free (s);
+    return ret;
+  }
 
   return g_strdup_printf ("%p", ptr);
 }
 
+#ifdef HAVE_PRINTF_EXTENSION
+
+static gchar *
+gst_debug_print_segment (gpointer ptr)
+{
+  GstSegment *segment = (GstSegment *) ptr;
+
+  /* nicely printed segment */
+  if (segment == NULL) {
+    return g_strdup ("(NULL)");
+  }
+
+  switch (segment->format) {
+    case GST_FORMAT_UNDEFINED:{
+      return g_strdup_printf ("UNDEFINED segment");
+    }
+    case GST_FORMAT_TIME:{
+      return g_strdup_printf ("time segment start=%" GST_TIME_FORMAT
+          ", stop=%" GST_TIME_FORMAT ", last_stop=%" GST_TIME_FORMAT
+          ", duration=%" GST_TIME_FORMAT ", rate=%f, applied_rate=%f"
+          ", flags=0x%02x, time=%" GST_TIME_FORMAT ", accum=%" GST_TIME_FORMAT,
+          GST_TIME_ARGS (segment->start), GST_TIME_ARGS (segment->stop),
+          GST_TIME_ARGS (segment->last_stop), GST_TIME_ARGS (segment->duration),
+          segment->rate, segment->applied_rate, (guint) segment->flags,
+          GST_TIME_ARGS (segment->time), GST_TIME_ARGS (segment->accum));
+    }
+    default:{
+      const gchar *format_name;
+
+      format_name = gst_format_get_name (segment->format);
+      if (G_UNLIKELY (format_name == NULL))
+        format_name = "(UNKNOWN FORMAT)";
+      return g_strdup_printf ("%s segment start=%" G_GINT64_FORMAT
+          ", stop=%" G_GINT64_FORMAT ", last_stop=%" G_GINT64_FORMAT
+          ", duration=%" G_GINT64_FORMAT ", rate=%f, applied_rate=%f"
+          ", flags=0x%02x, time=%" GST_TIME_FORMAT ", accum=%" GST_TIME_FORMAT,
+          format_name, segment->start, segment->stop, segment->last_stop,
+          segment->duration, segment->rate, segment->applied_rate,
+          (guint) segment->flags, GST_TIME_ARGS (segment->time),
+          GST_TIME_ARGS (segment->accum));
+    }
+  }
+}
+
+#endif /* HAVE_PRINTF_EXTENSION */
+
 /**
  * gst_debug_construct_term_color:
  * @colorinfo: the color info
@@ -556,11 +627,20 @@ gst_debug_log_default (GstDebugCategory * category, GstDebugLevel level,
   gchar *clear;
   gchar *obj = NULL;
   gchar pidcolor[10];
+  const gchar *levelcolor;
   gint pid;
   GTimeVal now;
   GstClockTime elapsed;
   gboolean free_color = TRUE;
   gboolean free_obj = TRUE;
+  static const gchar *levelcolormap[] = {
+    "\033[37m",                 /* GST_LEVEL_NONE */
+    "\033[31;01m",              /* GST_LEVEL_ERROR */
+    "\033[33;01m",              /* GST_LEVEL_WARNING */
+    "\033[32;01m",              /* GST_LEVEL_INFO */
+    "\033[36m",                 /* GST_LEVEL_DEBUG */
+    "\033[37m"                  /* GST_LEVEL_LOG */
+  };
 
   if (level > gst_debug_category_get_threshold (category))
     return;
@@ -573,11 +653,13 @@ gst_debug_log_default (GstDebugCategory * category, GstDebugLevel level,
         (category));
     clear = "\033[00m";
     g_sprintf (pidcolor, "\033[3%1dm", pid % 6 + 31);
+    levelcolor = levelcolormap[level];
   } else {
     color = "\0";
     free_color = FALSE;
     clear = "";
     pidcolor[0] = '\0';
+    levelcolor = "\0";
   }
 
   if (object) {
@@ -588,13 +670,22 @@ gst_debug_log_default (GstDebugCategory * category, GstDebugLevel level,
   }
 
   g_get_current_time (&now);
-  elapsed = GST_TIMEVAL_TO_TIME (now) - start_time;
-  g_printerr ("%s (%p - %" GST_TIME_FORMAT
-      ") %s%20s%s(%s%5d%s) %s%s(%d):%s:%s%s %s\n",
-      gst_debug_level_get_name (level), g_thread_self (),
-      GST_TIME_ARGS (elapsed), color,
-      gst_debug_category_get_name (category), clear, pidcolor, pid, clear,
-      color, file, line, function, obj, clear, gst_debug_message_get (message));
+  elapsed = GST_TIMEVAL_TO_TIME (now) - _priv_gst_info_start_time;
+
+  /*
+     g_printerr ("%s (%p - %" GST_TIME_FORMAT ") %s%20s%s(%s%5d%s) %s%s(%d):%s:%s%s %s\n",
+     gst_debug_level_get_name (level), g_thread_self (),
+     GST_TIME_ARGS (elapsed), color,
+     gst_debug_category_get_name (category), clear, pidcolor, pid, clear,
+     color, file, line, function, obj, clear, gst_debug_message_get (message));
+   */
+
+  g_printerr ("%" GST_TIME_FORMAT
+      " %s%5d%s %p %s%s%s %s%20s %s:%d:%s:%s%s %s\n", GST_TIME_ARGS (elapsed),
+      pidcolor, pid, clear, g_thread_self (), levelcolor,
+      gst_debug_level_get_name (level), clear, color,
+      gst_debug_category_get_name (category), file, line, function, obj, clear,
+      gst_debug_message_get (message));
 
   if (free_color)
     g_free (color);
@@ -606,7 +697,7 @@ gst_debug_log_default (GstDebugCategory * category, GstDebugLevel level,
  * gst_debug_level_get_name:
  * @level: the level to get the name for
  *
- * Get the string trepresentation of a debugging level
+ * Get the string representation of a debugging level
  *
  * Returns: the name
  */
@@ -669,7 +760,7 @@ gst_debug_add_log_function (GstLogFunction func, gpointer data)
 static gint
 gst_debug_compare_log_function_by_func (gconstpointer entry, gconstpointer func)
 {
-  gpointer entryfunc = ((LogFuncEntry *) entry)->func;
+  gpointer entryfunc = (gpointer) (((LogFuncEntry *) entry)->func);
 
   return (entryfunc < func) ? -1 : (entryfunc > func) ? 1 : 0;
 }
@@ -711,7 +802,7 @@ gst_debug_remove_with_compare_func (GCompareFunc func, gpointer data)
  * gst_debug_remove_log_function:
  * @func: the log function to remove
  *
- * Removes all registrered instances of the given logging functions.
+ * Removes all registered instances of the given logging functions.
  *
  * Returns: How many instances of the function were removed
  */
@@ -724,7 +815,7 @@ gst_debug_remove_log_function (GstLogFunction func)
 
   removals =
       gst_debug_remove_with_compare_func
-      (gst_debug_compare_log_function_by_func, func);
+      (gst_debug_compare_log_function_by_func, (gpointer) func);
   GST_DEBUG ("removed log function %p %d times from log function list", func,
       removals);
 
@@ -735,7 +826,7 @@ gst_debug_remove_log_function (GstLogFunction func)
  * gst_debug_remove_log_function_by_data:
  * @data: user data of the log function to remove
  *
- * Removes all registrered instances of log functions with the given user data.
+ * Removes all registered instances of log functions with the given user data.
  *
  * Returns: How many instances of the function were removed
  */
@@ -794,9 +885,9 @@ gst_debug_set_active (gboolean active)
 {
   __gst_debug_enabled = active;
   if (active)
-    __gst_debug_min = GST_LEVEL_NONE;
-  else
     __gst_debug_min = GST_LEVEL_COUNT;
+  else
+    __gst_debug_min = GST_LEVEL_NONE;
 }
 
 /**
@@ -945,7 +1036,8 @@ gst_debug_unset_threshold_for_name (const gchar * name)
 }
 
 GstDebugCategory *
-_gst_debug_category_new (gchar * name, guint color, gchar * description)
+_gst_debug_category_new (const gchar * name, guint color,
+    const gchar * description)
 {
   GstDebugCategory *cat;
 
@@ -1012,8 +1104,10 @@ gst_debug_category_set_threshold (GstDebugCategory * category,
 {
   g_return_if_fail (category != NULL);
 
-  if (level > __gst_debug_min)
+  if (level > __gst_debug_min) {
     __gst_debug_enabled = TRUE;
+    __gst_debug_min = level;
+  }
 
   gst_atomic_int_set (&category->threshold, level);
 }
@@ -1038,7 +1132,7 @@ gst_debug_category_reset_threshold (GstDebugCategory * category)
  * gst_debug_category_get_threshold:
  * @category: a #GstDebugCategory to get threshold of.
  *
- * Returns the threshold of a #GstCategory.
+ * Returns the threshold of a #GstDebugCategory.
  *
  * Returns: the #GstDebugLevel that is used as threshold.
  */
@@ -1124,40 +1218,44 @@ _gst_debug_nameof_funcptr (GstDebugFuncPtr ptr)
 /* This function MUST NOT return NULL */
      const gchar *_gst_debug_nameof_funcptr (GstDebugFuncPtr func)
 {
-  gpointer ptr = (gpointer) func;
   gchar *ptrname;
 
 #ifdef HAVE_DLADDR
-  Dl_info dlinfo;
+  Dl_info dl_info;
 #endif
 
-  if (__gst_function_pointers) {
-    g_static_mutex_lock (&__dbg_functions_mutex);
-    ptrname = g_hash_table_lookup (__gst_function_pointers, ptr);
+  if (G_UNLIKELY (func == NULL))
+    return "(NULL)";
+
+  g_static_mutex_lock (&__dbg_functions_mutex);
+  if (G_LIKELY (__gst_function_pointers)) {
+    ptrname = g_hash_table_lookup (__gst_function_pointers, (gpointer) func);
     g_static_mutex_unlock (&__dbg_functions_mutex);
-    if (ptrname)
+    if (G_LIKELY (ptrname))
       return ptrname;
+  } else {
+    g_static_mutex_unlock (&__dbg_functions_mutex);
   }
   /* we need to create an entry in the hash table for this one so we don't leak
    * the name */
 #ifdef HAVE_DLADDR
-  if (dladdr (ptr, &dlinfo) && dlinfo.dli_sname) {
-    gchar *name = g_strdup (dlinfo.dli_sname);
+  if (dladdr ((gpointer) func, &dl_info) && dl_info.dli_sname) {
+    gchar *name = g_strdup (dl_info.dli_sname);
 
-    _gst_debug_register_funcptr (ptr, name);
+    _gst_debug_register_funcptr (func, name);
     return name;
   } else
 #endif
   {
-    gchar *name = g_strdup_printf ("%p", ptr);
+    gchar *name = g_strdup_printf ("%p", (gpointer) func);
 
-    _gst_debug_register_funcptr (ptr, name);
+    _gst_debug_register_funcptr (func, name);
     return name;
   }
 }
 
 void
-_gst_debug_register_funcptr (GstDebugFuncPtr func, gchar * ptrname)
+_gst_debug_register_funcptr (GstDebugFuncPtr func, const gchar * ptrname)
 {
   gpointer ptr = (gpointer) func;
 
@@ -1166,14 +1264,16 @@ _gst_debug_register_funcptr (GstDebugFuncPtr func, gchar * ptrname)
   if (!__gst_function_pointers)
     __gst_function_pointers = g_hash_table_new (g_direct_hash, g_direct_equal);
   if (!g_hash_table_lookup (__gst_function_pointers, ptr))
-    g_hash_table_insert (__gst_function_pointers, ptr, ptrname);
+    g_hash_table_insert (__gst_function_pointers, ptr, (gpointer) ptrname);
 
   g_static_mutex_unlock (&__dbg_functions_mutex);
 }
 
+/*** PRINTF EXTENSIONS ********************************************************/
+
 #ifdef HAVE_PRINTF_EXTENSION
 static int
-_gst_info_printf_extension (FILE * stream, const struct printf_info *info,
+_gst_info_printf_extension_ptr (FILE * stream, const struct printf_info *info,
     const void *const *args)
 {
   char *buffer;
@@ -1187,7 +1287,26 @@ _gst_info_printf_extension (FILE * stream, const struct printf_info *info,
   len = fprintf (stream, "%*s", (info->left ? -info->width : info->width),
       buffer);
 
-  free (buffer);
+  g_free (buffer);
+  return len;
+}
+
+static int
+_gst_info_printf_extension_segment (FILE * stream,
+    const struct printf_info *info, const void *const *args)
+{
+  char *buffer;
+  int len;
+  void *ptr;
+
+  buffer = NULL;
+  ptr = *(void **) args[0];
+
+  buffer = gst_debug_print_segment (ptr);
+  len = fprintf (stream, "%*s", (info->left ? -info->width : info->width),
+      buffer);
+
+  g_free (buffer);
   return len;
 }
 
@@ -1262,7 +1381,7 @@ __cyg_profile_func_exit (void *this_fn, void *call_site)
 /**
  * gst_debug_print_stack_trace:
  *
- * If GST_ENABLE_FUNC_INSTRUMENTATION is defined a stacktracke is available for
+ * If GST_ENABLE_FUNC_INSTRUMENTATION is defined a stacktrace is available for
  * gstreamer code, which can be printed with this function.
  */
 void
@@ -1289,4 +1408,4 @@ gst_debug_print_stack_trace (void)
   /* nothing because it's compiled out */
 }
 
-#endif /* GST_ENABLE_FUNC_INTSTRUMENTATION */
+#endif /* GST_ENABLE_FUNC_INSTRUMENTATION */
