@@ -28,6 +28,21 @@
 #include <glib-object.h>
 #include <gst/gstconfig.h>
 
+
+#ifndef _MSC_VER
+#define IMPORT_SYMBOL
+#else /* _MSC_VER */
+#ifndef LIBGSTREAMER_EXPORTS
+#define IMPORT_SYMBOL __declspec(dllimport)
+#else
+#define IMPORT_SYMBOL 
+#endif
+#endif
+
+#ifndef M_PI
+#define M_PI  3.14159265358979323846
+#endif
+
 G_BEGIN_DECLS
 
 /**
@@ -201,7 +216,7 @@ struct _GstDebugCategory {
  */
 #ifndef GST_FUNCTION
 #if defined (__GNUC__)
-#  define GST_FUNCTION     ((const char*) (__PRETTY_FUNCTION__))
+#  define GST_FUNCTION     ((const char*) (__FUNCTION__))
 #elif defined (G_HAVE_ISO_VARARGS)
 #  define GST_FUNCTION     ((const char*) (__func__))
 #else
@@ -321,12 +336,7 @@ void            gst_debug_unset_threshold_for_name  (const gchar * name);
  * Declares a GstDebugCategory variable as extern. Use in header files.
  * This macro expands to nothing if debugging is disabled.
  */
-#ifndef WIN32
 #define GST_DEBUG_CATEGORY_EXTERN(cat) extern GstDebugCategory *cat
-#else
-#define GST_DEBUG_CATEGORY_EXTERN(cat) \
-  extern __declspec (dllimport) GstDebugCategory *cat;
-#endif
 
 /**
  * GST_DEBUG_CATEGORY_STATIC:
@@ -337,9 +347,9 @@ void            gst_debug_unset_threshold_for_name  (const gchar * name);
  */
 #define GST_DEBUG_CATEGORY_STATIC(cat) static GstDebugCategory *cat = NULL
 /* do not use this function, use the macros below */
-GstDebugCategory *_gst_debug_category_new	(gchar *	name,
-						 guint		color,
-						 gchar *	description);
+GstDebugCategory *_gst_debug_category_new (const gchar * name,
+                                           guint         color,
+                                           const gchar * description);
 /**
  * GST_DEBUG_CATEGORY_INIT:
  * @cat: the category to initialize.
@@ -408,7 +418,7 @@ extern gboolean			__gst_debug_enabled;
 
 /* since 0.10.7, the min debug level, used for quickly discarding debug
  * messages that fall under the threshold. */
-extern GstDebugLevel            __gst_debug_min; 
+extern IMPORT_SYMBOL GstDebugLevel            __gst_debug_min; 
 
 /**
  * GST_CAT_LEVEL_LOG:
@@ -441,21 +451,21 @@ static inline void
 GST_CAT_LEVEL_LOG_valist (GstDebugCategory * cat,
     GstDebugLevel level, gpointer object, const char *format, va_list varargs)
 {
-  gst_debug_log_valist (cat, level, "", "", 0, (GObject *) object, format,
-      varargs);
+  if (G_UNLIKELY (level <= __gst_debug_min)) {
+    gst_debug_log_valist (cat, level, "", "", 0, (GObject *) object, format,
+        varargs);
+  }
 }
 
 static inline void
 GST_CAT_LEVEL_LOG (GstDebugCategory * cat, GstDebugLevel level,
     gpointer object, const char *format, ...)
 {
-  if (G_UNLIKELY (level <= __gst_debug_min)) {						\
-    va_list varargs;
+  va_list varargs;
 
-    va_start (varargs, format);
-    GST_CAT_LEVEL_LOG_valist (cat, level, object, format, varargs);
-    va_end (varargs);
-  }
+  va_start (varargs, format);
+  GST_CAT_LEVEL_LOG_valist (cat, level, object, format, varargs);
+  va_end (varargs);
 }
 #endif
 #endif /* G_HAVE_ISO_VARARGS */
@@ -914,8 +924,10 @@ G_CONST_RETURN gchar *
 #define GST_DEBUG_FUNCPTR_NAME(ptr) \
   _gst_debug_nameof_funcptr((GstDebugFuncPtr)ptr)
 
+
 #else /* GST_DISABLE_GST_DEBUG */
 
+
 #if defined(__GNUC__) && __GNUC__ >= 3
 #  pragma GCC poison gst_debug_log
 #  pragma GCC poison gst_debug_log_valist
@@ -1135,6 +1147,7 @@ GST_LOG (const char *format, ...)
 
 #endif /* GST_DISABLE_GST_DEBUG */
 
+
 void gst_debug_print_stack_trace (void);
 
 G_END_DECLS
