@@ -27,8 +27,8 @@
  * @see_also: #GstElement, #GstBin, #GstClock, #GstBus
  *
  * A #GstPipeline is a special #GstBin used as the toplevel container for
- * the filter graph. The #GstPipeline will manage the selection and 
- * distribution of a global #GstClock as well as provide a #GstBus to the 
+ * the filter graph. The #GstPipeline will manage the selection and
+ * distribution of a global #GstClock as well as provide a #GstBus to the
  * application. It will also implement a default behavour for managing
  * seek events (see gst_element_seek()).
  *
@@ -36,7 +36,7 @@
  * the pipeline, use gst_object_unref() to free its resources including all
  * added #GstElement objects (if not otherwise referenced).
  *
- * Elements are added and removed from the pipeline using the #GstBin 
+ * Elements are added and removed from the pipeline using the #GstBin
  * methods like gst_bin_add() and gst_bin_remove() (see #GstBin).
  *
  * Before changing the state of the #GstPipeline (see #GstElement) a #GstBus
@@ -50,8 +50,8 @@
  *
  * When the #GstPipeline performs the PAUSED to PLAYING state change it will
  * select a clock for the elements. The clock selection algorithm will by
- * default select a clock provided by an element that is most upstream 
- * (closest to the source). For live pipelines (ones that return 
+ * default select a clock provided by an element that is most upstream
+ * (closest to the source). For live pipelines (ones that return
  * #GST_STATE_CHANGE_NO_PREROLL from the gst_element_set_state() call) this
  * will select the clock provided by the live source. For normal pipelines
  * this will select a clock provided by the sinks (most likely the audio
@@ -59,20 +59,20 @@
  *
  * The clock selection can be controlled with the gst_pipeline_use_clock()
  * method, which will enforce a given clock on the pipeline. With
- * gst_pipeline_auto_clock() the default clock selection algorithm can be 
+ * gst_pipeline_auto_clock() the default clock selection algorithm can be
  * restored.
  *
  * A #GstPipeline maintains a stream time for the elements. The stream
  * time is defined as the difference between the current clock time and
  * the base time. When the pipeline goes to READY or a flushing seek is
  * performed on it, the stream time is reset to 0. When the pipeline is
- * set from PLAYING to PAUSED, the current clock time is sampled and used to 
+ * set from PLAYING to PAUSED, the current clock time is sampled and used to
  * configure the base time for the elements when the pipeline is set
  * to PLAYING again. This default behaviour can be changed with the
- * gst_pipeline_set_new_stream_time() method. 
- * 
- * When sending a flushing seek event to a GstPipeline (see 
- * gst_element_seek()), it will make sure that the pipeline is properly 
+ * gst_pipeline_set_new_stream_time() method.
+ *
+ * When sending a flushing seek event to a GstPipeline (see
+ * gst_element_seek()), it will make sure that the pipeline is properly
  * PAUSED and resumed as well as set the new stream time to 0 when the
  * seek succeeded.
  *
@@ -120,6 +120,11 @@ struct _GstPipelinePrivate
 {
   /* with LOCK */
   gboolean auto_flush_bus;
+
+  /* when we need to update stream_time or clock when going back to
+   * PLAYING*/
+  gboolean update_stream_time;
+  gboolean update_clock;
 };
 
 
@@ -133,13 +138,12 @@ static void gst_pipeline_set_property (GObject * object, guint prop_id,
 static void gst_pipeline_get_property (GObject * object, guint prop_id,
     GValue * value, GParamSpec * pspec);
 
-static gboolean gst_pipeline_send_event (GstElement * element,
-    GstEvent * event);
-
 static GstClock *gst_pipeline_provide_clock_func (GstElement * element);
 static GstStateChangeReturn gst_pipeline_change_state (GstElement * element,
     GstStateChange transition);
 
+static void gst_pipeline_handle_message (GstBin * bin, GstMessage * message);
+
 static GstBinClass *parent_class = NULL;
 
 /* static guint gst_pipeline_signals[LAST_SIGNAL] = { 0 }; */
@@ -185,6 +189,7 @@ gst_pipeline_class_init (gpointer g_class, gpointer class_data)
 {
   GObjectClass *gobject_class = G_OBJECT_CLASS (g_class);
   GstElementClass *gstelement_class = GST_ELEMENT_CLASS (g_class);
+  GstBinClass *gstbin_class = GST_BIN_CLASS (g_class);
   GstPipelineClass *klass = GST_PIPELINE_CLASS (g_class);
 
   parent_class = g_type_class_peek_parent (klass);
@@ -226,11 +231,12 @@ gst_pipeline_class_init (gpointer g_class, gpointer class_data)
 
   gobject_class->dispose = GST_DEBUG_FUNCPTR (gst_pipeline_dispose);
 
-  gstelement_class->send_event = GST_DEBUG_FUNCPTR (gst_pipeline_send_event);
   gstelement_class->change_state =
       GST_DEBUG_FUNCPTR (gst_pipeline_change_state);
   gstelement_class->provide_clock =
       GST_DEBUG_FUNCPTR (gst_pipeline_provide_clock_func);
+  gstbin_class->handle_message =
+      GST_DEBUG_FUNCPTR (gst_pipeline_handle_message);
 }
 
 static void
@@ -311,90 +317,19 @@ gst_pipeline_get_property (GObject * object, guint prop_id,
   }
 }
 
-/* default pipeline seeking code:
- *
- * If the pipeline is PLAYING and a flushing seek is done, set
- * the pipeline to PAUSED before doing the seek.
- *
- * A flushing seek also resets the stream time to 0 so that when
- * we go back to PLAYING after the seek, the base_time is recalculated
- * and redistributed to the elements.
- */
-static gboolean
-do_pipeline_seek (GstElement * element, GstEvent * event)
+/* set the stream time to 0 */
+static void
+reset_stream_time (GstPipeline * pipeline)
 {
-  gdouble rate;
-  GstSeekFlags flags;
-  gboolean flush;
-  gboolean was_playing = FALSE;
-  gboolean res;
-
-  /* we are only interested in the FLUSH flag of the seek event. */
-  gst_event_parse_seek (event, &rate, NULL, &flags, NULL, NULL, NULL, NULL);
-
-  flush = flags & GST_SEEK_FLAG_FLUSH;
-
-  /* if flushing seek, get the current state */
-  if (flush) {
-    GstState state;
-
-    /* need to call _get_state() since a bin state is only updated
-     * with this call. */
-    gst_element_get_state (element, &state, NULL, 0);
-    was_playing = (state == GST_STATE_PLAYING);
-
-    if (was_playing) {
-      /* and PAUSE when the pipeline was PLAYING, we don't need
-       * to wait for the state change to complete since we are going
-       * to flush out any preroll sample anyway. */
-      gst_element_set_state (element, GST_STATE_PAUSED);
-    }
+  GST_OBJECT_LOCK (pipeline);
+  if (pipeline->stream_time != GST_CLOCK_TIME_NONE) {
+    GST_DEBUG_OBJECT (pipeline, "reset stream_time to 0");
+    pipeline->stream_time = 0;
+    pipeline->priv->update_stream_time = TRUE;
+  } else {
+    GST_DEBUG_OBJECT (pipeline, "application asked to not reset stream_time");
   }
-
-  /* let parent class implement the seek behaviour */
-  res = GST_ELEMENT_CLASS (parent_class)->send_event (element, event);
-
-  /* if flushing seek restore previous state */
-  if (flush) {
-    gboolean need_reset;
-
-    GST_OBJECT_LOCK (element);
-    need_reset = GST_PIPELINE (element)->stream_time != GST_CLOCK_TIME_NONE;
-    GST_OBJECT_UNLOCK (element);
-
-    /* need to reset the stream time to 0 after a successfull flushing seek, 
-     * unless the user explicitly disabled this behavior by setting stream 
-     * time to NONE */
-    if (need_reset && res)
-      gst_pipeline_set_new_stream_time (GST_PIPELINE (element), 0);
-
-    if (was_playing)
-      /* and continue playing, this might return ASYNC in which case the
-       * application can wait for the PREROLL to complete after the seek. 
-       */
-      gst_element_set_state (element, GST_STATE_PLAYING);
-  }
-  return res;
-}
-
-static gboolean
-gst_pipeline_send_event (GstElement * element, GstEvent * event)
-{
-  gboolean res;
-  GstEventType event_type = GST_EVENT_TYPE (event);
-
-  switch (event_type) {
-    case GST_EVENT_SEEK:
-      /* do the default seek handling */
-      res = do_pipeline_seek (element, event);
-      break;
-    default:
-      /* else parent implements the defaults */
-      res = GST_ELEMENT_CLASS (parent_class)->send_event (element, event);
-      break;
-  }
-
-  return res;
+  GST_OBJECT_UNLOCK (pipeline);
 }
 
 /**
@@ -429,64 +364,96 @@ gst_pipeline_change_state (GstElement * element, GstStateChange transition)
       GST_OBJECT_UNLOCK (element);
       break;
     case GST_STATE_CHANGE_READY_TO_PAUSED:
+      GST_OBJECT_LOCK (element);
+      pipeline->priv->update_clock = TRUE;
+      GST_OBJECT_UNLOCK (element);
       break;
     case GST_STATE_CHANGE_PAUSED_TO_PLAYING:
     {
       GstClockTime new_base_time;
       GstClockTime start_time, stream_time, delay;
-      gboolean new_clock;
+      gboolean new_clock, update_stream_time, update_clock;
+      GstClock *cur_clock;
 
       GST_DEBUG_OBJECT (element, "selecting clock and base_time");
 
-      /* when going to playing, select a clock */
-      clock = gst_element_provide_clock (element);
-
-      if (clock) {
-        start_time = gst_clock_get_time (clock);
-      } else {
-        GST_DEBUG ("no clock, using base time of NONE");
-        start_time = GST_CLOCK_TIME_NONE;
-        new_base_time = GST_CLOCK_TIME_NONE;
-      }
-
       GST_OBJECT_LOCK (element);
-      new_clock = element->clock != clock;
+      cur_clock = element->clock;
+      if (cur_clock)
+        gst_object_ref (cur_clock);
       stream_time = pipeline->stream_time;
+      update_stream_time = pipeline->priv->update_stream_time;
+      update_clock = pipeline->priv->update_clock;
+      pipeline->priv->update_stream_time = FALSE;
+      pipeline->priv->update_clock = FALSE;
       delay = pipeline->delay;
       GST_OBJECT_UNLOCK (element);
 
-      if (new_clock) {
-        /* now distribute the clock (which could be NULL). If some
-         * element refuses the clock, this will return FALSE and
-         * we effectively fail the state change. */
-        if (!gst_element_set_clock (element, clock))
-          goto invalid_clock;
+      /* stream time changed, either with a PAUSED or a flush, we need to check
+       * if there is a new clock & update the base time */
+      if (update_stream_time) {
+        GST_DEBUG_OBJECT (pipeline, "Need to update stream_time");
 
-        /* if we selected and distributed a new clock, let the app 
-         * know about it */
-        gst_element_post_message (element,
-            gst_message_new_new_clock (GST_OBJECT_CAST (element), clock));
+        /* when going to PLAYING, select a clock when needed. If we just got
+         * flushed, we don't reselect the clock. */
+        if (update_clock) {
+          GST_DEBUG_OBJECT (pipeline, "Need to update clock.");
+          clock = gst_element_provide_clock (element);
+        } else {
+          GST_DEBUG_OBJECT (pipeline,
+              "Don't need to update clock, using old clock.");
+          clock = gst_object_ref (cur_clock);
+        }
+
+        new_clock = (clock != cur_clock);
+
+        if (clock) {
+          start_time = gst_clock_get_time (clock);
+        } else {
+          GST_DEBUG ("no clock, using base time of NONE");
+          start_time = GST_CLOCK_TIME_NONE;
+          new_base_time = GST_CLOCK_TIME_NONE;
+        }
+
+        if (new_clock) {
+          /* now distribute the clock (which could be NULL). If some
+           * element refuses the clock, this will return FALSE and
+           * we effectively fail the state change. */
+          if (!gst_element_set_clock (element, clock))
+            goto invalid_clock;
+
+          /* if we selected and distributed a new clock, let the app
+           * know about it */
+          gst_element_post_message (element,
+              gst_message_new_new_clock (GST_OBJECT_CAST (element), clock));
+        }
+
+        if (clock)
+          gst_object_unref (clock);
+
+        if (stream_time != GST_CLOCK_TIME_NONE
+            && start_time != GST_CLOCK_TIME_NONE) {
+          new_base_time = start_time - stream_time + delay;
+          GST_DEBUG_OBJECT (element,
+              "stream_time=%" GST_TIME_FORMAT ", now=%" GST_TIME_FORMAT
+              ", base_time %" GST_TIME_FORMAT,
+              GST_TIME_ARGS (stream_time), GST_TIME_ARGS (start_time),
+              GST_TIME_ARGS (new_base_time));
+        } else
+          new_base_time = GST_CLOCK_TIME_NONE;
+
+        if (new_base_time != GST_CLOCK_TIME_NONE)
+          gst_element_set_base_time (element, new_base_time);
+        else
+          GST_DEBUG_OBJECT (pipeline,
+              "NOT adjusting base_time because stream_time is NONE");
+      } else {
+        GST_DEBUG_OBJECT (pipeline,
+            "NOT adjusting base_time because we selected one before");
       }
 
-      if (stream_time != GST_CLOCK_TIME_NONE
-          && start_time != GST_CLOCK_TIME_NONE) {
-        new_base_time = start_time - stream_time + delay;
-        GST_DEBUG_OBJECT (element,
-            "stream_time=%" GST_TIME_FORMAT ", now=%" GST_TIME_FORMAT
-            ", base_time %" GST_TIME_FORMAT,
-            GST_TIME_ARGS (stream_time), GST_TIME_ARGS (start_time),
-            GST_TIME_ARGS (new_base_time));
-      } else
-        new_base_time = GST_CLOCK_TIME_NONE;
-
-      if (clock)
-        gst_object_unref (clock);
-
-      if (new_base_time != GST_CLOCK_TIME_NONE)
-        gst_element_set_base_time (element, new_base_time);
-      else
-        GST_DEBUG_OBJECT (pipeline,
-            "NOT adjusting base time because stream time is NONE");
+      if (cur_clock)
+        gst_object_unref (cur_clock);
       break;
     }
     case GST_STATE_CHANGE_PLAYING_TO_PAUSED:
@@ -503,17 +470,7 @@ gst_pipeline_change_state (GstElement * element, GstStateChange transition)
       break;
     case GST_STATE_CHANGE_READY_TO_PAUSED:
     {
-      gboolean need_reset;
-
-      /* only reset the stream_time when the application did not
-       * specify a stream_time explicitly */
-      GST_OBJECT_LOCK (element);
-      need_reset = pipeline->stream_time != GST_CLOCK_TIME_NONE;
-      GST_OBJECT_UNLOCK (element);
-
-      if (need_reset)
-        gst_pipeline_set_new_stream_time (pipeline, 0);
-
+      reset_stream_time (pipeline);
       break;
     }
     case GST_STATE_CHANGE_PAUSED_TO_PLAYING:
@@ -532,8 +489,14 @@ gst_pipeline_change_state (GstElement * element, GstStateChange transition)
 
         GST_OBJECT_LOCK (element);
         /* store the current stream time */
-        if (pipeline->stream_time != GST_CLOCK_TIME_NONE)
+        if (pipeline->stream_time != GST_CLOCK_TIME_NONE) {
           pipeline->stream_time = now - element->base_time;
+          /* we went to PAUSED, when going to PLAYING select clock and new
+           * base_time */
+          pipeline->priv->update_stream_time = TRUE;
+          pipeline->priv->update_clock = TRUE;
+        }
+
         GST_DEBUG_OBJECT (element,
             "stream_time=%" GST_TIME_FORMAT ", now=%" GST_TIME_FORMAT
             ", base_time %" GST_TIME_FORMAT,
@@ -574,6 +537,31 @@ invalid_clock:
   }
 }
 
+static void
+gst_pipeline_handle_message (GstBin * bin, GstMessage * message)
+{
+  GstPipeline *pipeline = GST_PIPELINE_CAST (bin);
+
+  switch (GST_MESSAGE_TYPE (message)) {
+    case GST_MESSAGE_ASYNC_START:
+    {
+      gboolean new_base_time;
+
+      gst_message_parse_async_start (message, &new_base_time);
+
+      /* reset our stream time if we need to distribute a new base_time to the
+       * children. */
+      if (new_base_time)
+        reset_stream_time (pipeline);
+
+      break;
+    }
+    default:
+      break;
+  }
+  GST_BIN_CLASS (parent_class)->handle_message (bin, message);
+}
+
 /**
  * gst_pipeline_get_bus:
  * @pipeline: a #GstPipeline
@@ -614,6 +602,7 @@ gst_pipeline_set_new_stream_time (GstPipeline * pipeline, GstClockTime time)
 
   GST_OBJECT_LOCK (pipeline);
   pipeline->stream_time = time;
+  pipeline->priv->update_stream_time = TRUE;
   GST_OBJECT_UNLOCK (pipeline);
 
   GST_DEBUG_OBJECT (pipeline, "set new stream_time to %" GST_TIME_FORMAT,
@@ -629,11 +618,11 @@ gst_pipeline_set_new_stream_time (GstPipeline * pipeline, GstClockTime time)
  *
  * Gets the last stream time of @pipeline. If the pipeline is PLAYING,
  * the returned time is the stream time used to configure the element's
- * base time in the PAUSED->PLAYING state. If the pipeline is PAUSED, the 
+ * base time in the PAUSED->PLAYING state. If the pipeline is PAUSED, the
  * returned time is the stream time when the pipeline was paused.
  *
  * This function returns #GST_CLOCK_TIME_NONE if the pipeline was
- * configured to not handle the management of the element's base time 
+ * configured to not handle the management of the element's base time
  * (see gst_pipeline_set_new_stream_time()).
  *
  * Returns: a #GstClockTime.
@@ -745,7 +734,7 @@ gst_pipeline_use_clock (GstPipeline * pipeline, GstClock * clock)
  * @clock: the clock to set
  *
  * Set the clock for @pipeline. The clock will be distributed
- * to all the elements managed by the pipeline. 
+ * to all the elements managed by the pipeline.
  *
  * Returns: TRUE if the clock could be set on the pipeline. FALSE if
  *   some element did not accept the clock.
@@ -767,9 +756,9 @@ gst_pipeline_set_clock (GstPipeline * pipeline, GstClock * clock)
  * @pipeline: a #GstPipeline
  *
  * Let @pipeline select a clock automatically. This is the default
- * behaviour. 
+ * behaviour.
  *
- * Use this function if you previous forced a fixed clock with 
+ * Use this function if you previous forced a fixed clock with
  * gst_pipeline_use_clock() and want to restore the default
  * pipeline clock selection algorithm.
  *
@@ -804,7 +793,7 @@ gst_pipeline_auto_clock (GstPipeline * pipeline)
  * amount of time before starting to process buffers and cannot be
  * #GST_CLOCK_TIME_NONE.
  *
- * This option is used for tuning purposes and should normally not be 
+ * This option is used for tuning purposes and should normally not be
  * used.
  *
  * MT safe.
@@ -856,15 +845,15 @@ gst_pipeline_get_delay (GstPipeline * pipeline)
  *
  * Usually, when a pipeline goes from READY to NULL state, it automatically
  * flushes all pending messages on the bus, which is done for refcounting
- * purposes, to break circular references. 
+ * purposes, to break circular references.
  *
- * This means that applications that update state using (async) bus messages 
- * (e.g. do certain things when a pipeline goes from PAUSED to READY) might 
- * not get to see messages when the pipeline is shut down, because they might 
+ * This means that applications that update state using (async) bus messages
+ * (e.g. do certain things when a pipeline goes from PAUSED to READY) might
+ * not get to see messages when the pipeline is shut down, because they might
  * be flushed before they can be dispatched in the main thread. This behaviour
  * can be disabled using this function.
  *
- * It is important that all messages on the bus are handled when the 
+ * It is important that all messages on the bus are handled when the
  * automatic flushing is disabled else memory leaks will be introduced.
  *
  * MT safe.
