@@ -29,11 +29,12 @@
  * A plugin system can provide one or more of the basic
  * <application>GStreamer</application> #GstPluginFeature subclasses.
  *
- * A plugin should export a symbol <symbol>plugin_desc</symbol> that is a
+ * A plugin should export a symbol <symbol>gst_plugin_desc</symbol> that is a
  * struct of type #GstPluginDesc.
  * the plugin loader will check the version of the core library the plugin was
  * linked against and will create a new #GstPlugin. It will then call the
- * #GstPluginInitFunc function that was provided in the plugin_desc.
+ * #GstPluginInitFunc function that was provided in the
+ * <symbol>gst_plugin_desc</symbol>.
  *
  * Once you have a handle to a #GstPlugin (e.g. from the #GstRegistryPool), you
  * can add any object that subclasses #GstPluginFeature.
@@ -51,8 +52,8 @@
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
+#include <glib/gstdio.h>
 #include <sys/types.h>
-#include <sys/stat.h>
 #ifdef HAVE_DIRENT_H
 #include <dirent.h>
 #endif
@@ -64,22 +65,15 @@
 
 #include "gst_private.h"
 
-#include "gstplugin.h"
-#include "gstversion.h"
-#include "gstinfo.h"
-#include "gstfilter.h"
-#include "gstregistry.h"
-#include "gstmacros.h"
-
+#include <gst/gst.h>
 
 #define GST_CAT_DEFAULT GST_CAT_PLUGIN_LOADING
 
-static GModule *main_module = NULL;
 static GList *_gst_plugin_static = NULL;
+static gboolean _gst_plugin_inited;
 
 /* static variables for segfault handling of plugin loading */
 static char *_gst_plugin_fault_handler_filename = NULL;
-extern gboolean _gst_disable_segtrap;   /* see gst.c */
 
 #ifndef HAVE_WIN32
 static gboolean _gst_plugin_fault_handler_is_setup = FALSE;
@@ -94,6 +88,8 @@ static gboolean _gst_plugin_fault_handler_is_setup = FALSE;
  * LGPL: http://www.gnu.org/copyleft/lesser.html
  * QPL: http://www.trolltech.com/licenses/qpl.html
  * MPL: http://www.opensource.org/licenses/mozilla1.1.php
+ * MIT/X11: http://www.opensource.org/licenses/mit-license.php
+ * 3-clause BSD: http://www.opensource.org/licenses/bsd-license.php
  */
 static const gchar *valid_licenses[] = {
   "LGPL",                       /* GNU Lesser General Public License */
@@ -101,15 +97,17 @@ static const gchar *valid_licenses[] = {
   "QPL",                        /* Trolltech Qt Public License */
   "GPL/QPL",                    /* Combi-license of GPL + QPL */
   "MPL",                        /* MPL 1.1 license */
+  "BSD",                        /* 3-clause BSD license */
+  "MIT/X11",                    /* MIT/X11 license */
   "Proprietary",                /* Proprietary license */
   GST_LICENSE_UNKNOWN,          /* some other license */
   NULL
 };
 
 static GstPlugin *gst_plugin_register_func (GstPlugin * plugin,
-    GModule * module, GstPluginDesc * desc);
-static void
-gst_plugin_desc_copy (GstPluginDesc * dest, const GstPluginDesc * src);
+    GstPluginDesc * desc);
+static void gst_plugin_desc_copy (GstPluginDesc * dest,
+    const GstPluginDesc * src);
 static void gst_plugin_desc_free (GstPluginDesc * desc);
 
 
@@ -118,7 +116,7 @@ G_DEFINE_TYPE (GstPlugin, gst_plugin, GST_TYPE_OBJECT);
 static void
 gst_plugin_init (GstPlugin * plugin)
 {
-
+  /* do nothing, needed because of G_DEFINE_TYPE */
 }
 
 static void
@@ -166,7 +164,7 @@ gst_plugin_error_quark (void)
 void
 _gst_plugin_register_static (GstPluginDesc * desc)
 {
-  if (main_module == NULL) {
+  if (!_gst_plugin_inited) {
     if (GST_CAT_DEFAULT)
       GST_LOG ("queueing static plugin \"%s\" for loading later on",
           desc->name);
@@ -177,7 +175,7 @@ _gst_plugin_register_static (GstPluginDesc * desc)
     if (GST_CAT_DEFAULT)
       GST_LOG ("attempting to load static plugin \"%s\" now...", desc->name);
     plugin = g_object_new (GST_TYPE_PLUGIN, NULL);
-    if (gst_plugin_register_func (plugin, main_module, desc)) {
+    if (gst_plugin_register_func (plugin, desc)) {
       if (GST_CAT_DEFAULT)
         GST_INFO ("loaded static plugin \"%s\"", desc->name);
       gst_default_registry_add_plugin (plugin);
@@ -188,7 +186,7 @@ _gst_plugin_register_static (GstPluginDesc * desc)
 void
 _gst_plugin_initialize (void)
 {
-  main_module = g_module_open (NULL, G_MODULE_BIND_LAZY);
+  _gst_plugin_inited = TRUE;
 
   /* now register all static plugins */
   g_list_foreach (_gst_plugin_static, (GFunc) _gst_plugin_register_static,
@@ -226,8 +224,7 @@ gst_plugin_check_version (gint major, gint minor)
 }
 
 static GstPlugin *
-gst_plugin_register_func (GstPlugin * plugin, GModule * module,
-    GstPluginDesc * desc)
+gst_plugin_register_func (GstPlugin * plugin, GstPluginDesc * desc)
 {
   if (!gst_plugin_check_version (desc->major_version, desc->minor_version)) {
     if (GST_CAT_DEFAULT)
@@ -269,7 +266,7 @@ gst_plugin_register_func (GstPlugin * plugin, GModule * module,
   return plugin;
 }
 
-#ifndef HAVE_WIN32
+#ifdef HAVE_SIGACTION
 static struct sigaction oldaction;
 
 /*
@@ -323,7 +320,7 @@ _gst_plugin_fault_handler_setup (void)
   struct sigaction action;
 
   /* if asked to leave segfaults alone, just return */
-  if (_gst_disable_segtrap)
+  if (!gst_segtrap_is_enabled ())
     return;
 
   if (_gst_plugin_fault_handler_is_setup)
@@ -350,7 +347,7 @@ _gst_plugin_fault_handler_setup (void)
 
 static void _gst_plugin_fault_handler_setup ();
 
-GStaticMutex gst_plugin_loading_mutex = G_STATIC_MUTEX_INIT;
+static GStaticMutex gst_plugin_loading_mutex = G_STATIC_MUTEX_INIT;
 
 /**
  * gst_plugin_load_file:
@@ -399,7 +396,7 @@ gst_plugin_load_file (const gchar * filename, GError ** error)
     goto return_error;
   }
 
-  if (stat (filename, &file_status)) {
+  if (g_stat (filename, &file_status)) {
     GST_CAT_DEBUG (GST_CAT_PLUGIN_LOADING, "problem accessing file");
     g_set_error (error,
         GST_PLUGIN_ERROR,
@@ -413,7 +410,12 @@ gst_plugin_load_file (const gchar * filename, GError ** error)
     GST_CAT_WARNING (GST_CAT_PLUGIN_LOADING, "module_open failed: %s",
         g_module_error ());
     g_set_error (error,
-        GST_PLUGIN_ERROR, GST_PLUGIN_ERROR_MODULE, "Opening module failed");
+        GST_PLUGIN_ERROR, GST_PLUGIN_ERROR_MODULE, "Opening module failed: %s",
+        g_module_error ());
+    /* If we failed to open the shared object, then it's probably because a
+     * plugin is linked against the wrong libraries. Print out an easy-to-see
+     * message in this case. */
+    g_warning ("Failed to load plugin '%s': %s", filename, g_module_error ());
     goto return_error;
   }
 
@@ -447,7 +449,7 @@ gst_plugin_load_file (const gchar * filename, GError ** error)
   GST_LOG ("Plugin %p for file \"%s\" prepared, registering...",
       plugin, filename);
 
-  if (!gst_plugin_register_func (plugin, module, plugin->orig_desc)) {
+  if (!gst_plugin_register_func (plugin, plugin->orig_desc)) {
     /* remove signal handler */
     _gst_plugin_fault_handler_restore ();
     GST_DEBUG ("gst_plugin_register_func failed for plugin \"%s\"", filename);
@@ -486,28 +488,22 @@ gst_plugin_desc_copy (GstPluginDesc * dest, const GstPluginDesc * src)
 {
   dest->major_version = src->major_version;
   dest->minor_version = src->minor_version;
-  dest->name = g_strdup (src->name);
+  dest->name = g_intern_string (src->name);
+  /* maybe intern the description too, just for convenience? */
   dest->description = g_strdup (src->description);
   dest->plugin_init = src->plugin_init;
-  dest->version = g_strdup (src->version);
-  dest->license = g_strdup (src->license);
-  dest->source = g_strdup (src->source);
-  dest->package = g_strdup (src->package);
-  dest->origin = g_strdup (src->origin);
+  dest->version = g_intern_string (src->version);
+  dest->license = g_intern_string (src->license);
+  dest->source = g_intern_string (src->source);
+  dest->package = g_intern_string (src->package);
+  dest->origin = g_intern_string (src->origin);
 }
 
 /* unused */
 static void
 gst_plugin_desc_free (GstPluginDesc * desc)
 {
-  g_free (desc->name);
   g_free (desc->description);
-  g_free (desc->version);
-  g_free (desc->license);
-  g_free (desc->source);
-  g_free (desc->package);
-  g_free (desc->origin);
-
   memset (desc, 0, sizeof (GstPluginDesc));
 }
 
@@ -875,7 +871,7 @@ gst_plugin_load_by_name (const gchar * name)
     gst_object_unref (plugin);
 
     if (!newplugin) {
-      GST_WARNING ("load_plugin error: %s\n", error->message);
+      GST_WARNING ("load_plugin error: %s", error->message);
       g_error_free (error);
       return NULL;
     }
@@ -921,7 +917,7 @@ gst_plugin_load (GstPlugin * plugin)
 
 load_error:
   {
-    GST_WARNING ("load_plugin error: %s\n", error->message);
+    GST_WARNING ("load_plugin error: %s", error->message);
     g_error_free (error);
     return NULL;
   }
