@@ -101,21 +101,23 @@ typedef gboolean (*GstPluginInitFunc) (GstPlugin *plugin);
  * @origin: URL to provider of plugin
  * @_gst_reserved: private, for later expansion
  *
+ * A plugin should export a variable of this type called plugin_desc. The plugin
+ * loader will use the data provided there to initialize the plugin.
  *
- * A plugins should export a variable of this type called plugin_desc. This plugin
- * loaded will use this variable to initialize the plugin.
+ * The @licence parameter must be one of: LGPL, GPL, QPL, GPL/QPL, MPL,
+ * BSD, MIT/X11, Proprietary, unknown.
  */
 struct _GstPluginDesc {
   gint major_version;
   gint minor_version;
-  gchar *name;
+  const gchar *name;
   gchar *description;
   GstPluginInitFunc plugin_init;
-  gchar *version;
-  gchar *license;
-  gchar *source;
-  gchar *package;
-  gchar *origin;
+  const gchar *version;
+  const gchar *license;
+  const gchar *source;
+  const gchar *package;
+  const gchar *origin;
 
   gpointer _gst_reserved[GST_PADDING];
 };
@@ -185,7 +187,7 @@ GST_PLUGIN_EXPORT GstPluginDesc gst_plugin_desc = {	\
   major,						\
   minor,						\
   name,							\
-  description,						\
+  (gchar *) description,				\
   init,							\
   version,						\
   license,						\
@@ -219,7 +221,7 @@ _gst_plugin_static_init__ ##init (void)			\
     major,						\
     minor,						\
     name,						\
-    description,					\
+    (gchar *) description,				\
     init,						\
     version,						\
     license,						\
@@ -256,7 +258,6 @@ typedef gboolean        (*GstPluginFilter)              (GstPlugin *plugin,
 
 GType                   gst_plugin_get_type             (void);
 
-void			_gst_plugin_initialize		(void);
 void			_gst_plugin_register_static	(GstPluginDesc *desc);
 
 G_CONST_RETURN gchar*	gst_plugin_get_name		(GstPlugin *plugin);
