@@ -69,7 +69,6 @@
 GST_DEBUG_CATEGORY_STATIC (gst_query_debug);
 #define GST_CAT_DEFAULT gst_query_debug
 
-static void gst_query_init (GTypeInstance * instance, gpointer g_class);
 static void gst_query_class_init (gpointer g_class, gpointer class_data);
 static void gst_query_finalize (GstQuery * query);
 static GstQuery *_gst_query_copy (GstQuery * query);
@@ -120,7 +119,7 @@ _gst_query_initialize (void)
   }
   g_static_mutex_unlock (&mutex);
 
-  gst_query_get_type ();
+  g_type_class_ref (gst_query_get_type ());
 }
 
 /**
@@ -174,7 +173,7 @@ gst_query_get_type (void)
       NULL,
       sizeof (GstQuery),
       0,
-      gst_query_init,
+      NULL,
       NULL
     };
 
@@ -207,12 +206,6 @@ gst_query_finalize (GstQuery * query)
   }
 }
 
-static void
-gst_query_init (GTypeInstance * instance, gpointer g_class)
-{
-
-}
-
 static GstQuery *
 _gst_query_copy (GstQuery * query)
 {
