@@ -1,7 +1,7 @@
 /* GStreamer
  * Copyright (C) 2006 Josep Torra <josep@fluendo.com>
  *               2006 Mathieu Garcia <matthieu@fluendo.com>
- *		         2006,2007 Stefan Kost <ensonic@users.sf.net>
+ *               2006,2007 Stefan Kost <ensonic@users.sf.net>
  *
  * gstregistrybinary.c: GstRegistryBinary object, support routines
  *
@@ -24,9 +24,14 @@
 /* FIXME:
  * - Add random key to libgstreamer during build and only accept registry,
  *   if key matches (or is the version check enough)
- * - handle cases where we can't mmap
  * - keep registry binary blob and reference strings
- *   (need const flags in GstPlugin, etc.)
+ *   - don't free/unmmap contents when leaving gst_registry_binary_read_cache()
+ *     - free at gst_deinit() / _priv_gst_registry_cleanup() ?
+ *   - GstPlugin:
+ *     - GST_PLUGIN_FLAG_CONST
+ *   -GstPluginFeature, GstIndexFactory, GstElementFactory
+ *     - needs Flags (GST_PLUGIN_FEATURE_FLAG_CONST)
+ *     - can we turn loaded into flag?
  * - why do we collect a list of binary chunks and not write immediately
  *   - because we need to process subchunks, before we can set e.g. nr_of_items
  *     in parent chunk
@@ -35,8 +40,34 @@
  *   - include md5-sum ?
  */
 
+#ifdef HAVE_CONFIG_H
+#  include "config.h"
+#endif
+
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
+
+#include <errno.h>
+#include <stdio.h>
+
+#include <gst/gst_private.h>
+#include <gst/gstconfig.h>
+#include <gst/gstelement.h>
+#include <gst/gsttypefind.h>
+#include <gst/gsttypefindfactory.h>
+#include <gst/gsturi.h>
+#include <gst/gstinfo.h>
+#include <gst/gstenumtypes.h>
+#include <gst/gstpadtemplate.h>
+
 #include <gst/gstregistrybinary.h>
 
+#include <glib/gstdio.h>        /* for g_stat(), g_mapped_file(), ... */
+
+#include "glib-compat-private.h"
+
+
 #define GST_CAT_DEFAULT GST_CAT_REGISTRY
 
 /* macros */
@@ -45,6 +76,10 @@
   _outptr = (_element *)_inptr; \
   _inptr += sizeof (_element)
 
+#define unpack_const_string(_inptr, _outptr) \
+  _outptr = g_intern_string ((const gchar *)_inptr); \
+  _inptr += strlen(_outptr) + 1
+
 #define unpack_string(_inptr, _outptr)  \
   _outptr = g_strdup ((gchar *)_inptr); \
   _inptr += strlen(_outptr) + 1
@@ -57,6 +92,7 @@
 #  define align32(_ptr)          do {} while(0)
 #endif
 
+
 /* Registry saving */
 
 /*
@@ -93,10 +129,11 @@ gst_registry_binary_write (GstRegistry * registry, const void *mem,
   return TRUE;
 }
 
+
 /*
  * gst_registry_binary_initialize_magic:
  *
- * Initialize the GstBinaryRegistryMagic, setting both our magic number and 
+ * Initialize the GstBinaryRegistryMagic, setting both our magic number and
  * gstreamer major/minor version
  */
 inline static gboolean
@@ -111,6 +148,28 @@ gst_registry_binary_initialize_magic (GstBinaryRegistryMagic * m)
   return TRUE;
 }
 
+
+/*
+ * gst_registry_binary_save_const_string:
+ *
+ * Store a const string in a binary chunk.
+ *
+ * Returns: %TRUE for success
+ */
+inline static gboolean
+gst_registry_binary_save_const_string (GList ** list, const gchar * str)
+{
+  GstBinaryChunk *chunk;
+
+  chunk = g_malloc (sizeof (GstBinaryChunk));
+  chunk->data = (gpointer) str;
+  chunk->size = strlen ((gchar *) chunk->data) + 1;
+  chunk->flags = GST_BINARY_REGISTRY_FLAG_CONST;
+  chunk->align = FALSE;
+  *list = g_list_prepend (*list, chunk);
+  return TRUE;
+}
+
 /*
  * gst_registry_binary_save_string:
  *
@@ -126,12 +185,13 @@ gst_registry_binary_save_string (GList ** list, gchar * str)
   chunk = g_malloc (sizeof (GstBinaryChunk));
   chunk->data = str;
   chunk->size = strlen ((gchar *) chunk->data) + 1;
-  chunk->flags = GST_BINARY_REGISTRY_FLAG_CONST;
+  chunk->flags = GST_BINARY_REGISTRY_FLAG_NONE;
   chunk->align = FALSE;
   *list = g_list_prepend (*list, chunk);
   return TRUE;
 }
 
+
 /*
  * gst_registry_binary_save_data:
  *
@@ -152,6 +212,7 @@ gst_registry_binary_make_data (gpointer data, gulong size)
   return chunk;
 }
 
+
 /*
  * gst_registry_binary_save_pad_template:
  *
@@ -173,15 +234,16 @@ gst_registry_binary_save_pad_template (GList ** list,
   pt->direction = template->direction;
 
   /* pack pad template strings */
-  gst_registry_binary_save_string (list,
+  gst_registry_binary_save_const_string (list,
       (gchar *) (template->static_caps.string));
-  gst_registry_binary_save_string (list, template->name_template);
+  gst_registry_binary_save_const_string (list, template->name_template);
 
   *list = g_list_prepend (*list, chk);
 
   return TRUE;
 }
 
+
 /*
  * gst_registry_binary_save_feature:
  *
@@ -193,8 +255,8 @@ static gboolean
 gst_registry_binary_save_feature (GList ** list, GstPluginFeature * feature)
 {
   const gchar *type_name = g_type_name (G_OBJECT_TYPE (feature));
-  GstBinaryPluginFeature *pf;
-  GstBinaryChunk *chk;
+  GstBinaryPluginFeature *pf = NULL;
+  GstBinaryChunk *chk = NULL;
   GList *walk;
 
   if (!type_name) {
@@ -202,21 +264,21 @@ gst_registry_binary_save_feature (GList ** list, GstPluginFeature * feature)
     return FALSE;
   }
 
-  pf = g_malloc (sizeof (GstBinaryPluginFeature));
-  chk = gst_registry_binary_make_data (pf, sizeof (GstBinaryPluginFeature));
-
-  pf->rank = feature->rank;
-  pf->npadtemplates = pf->ninterfaces = pf->nuriprotocols = 0;
-
   if (GST_IS_ELEMENT_FACTORY (feature)) {
+    GstBinaryElementFactory *ef;
     GstElementFactory *factory = GST_ELEMENT_FACTORY (feature);
 
+    ef = g_malloc (sizeof (GstBinaryElementFactory));
+    chk = gst_registry_binary_make_data (ef, sizeof (GstBinaryElementFactory));
+    ef->npadtemplates = ef->ninterfaces = ef->nuriprotocols = 0;
+    pf = (GstBinaryPluginFeature *) ef;
+
     /* save interfaces */
     for (walk = factory->interfaces; walk;
-        walk = g_list_next (walk), pf->ninterfaces++) {
-      gst_registry_binary_save_string (list, (gchar *) walk->data);
+        walk = g_list_next (walk), ef->ninterfaces++) {
+      gst_registry_binary_save_const_string (list, (gchar *) walk->data);
     }
-    GST_DEBUG ("Saved %d Interfaces", pf->ninterfaces);
+    GST_DEBUG ("Saved %d Interfaces", ef->ninterfaces);
     /* save uritypes */
     if (GST_URI_TYPE_IS_VALID (factory->uri_type)) {
       if (factory->uri_protocols) {
@@ -230,11 +292,11 @@ gst_registry_binary_save_feature (GList ** list, GstPluginFeature * feature)
 
         protocol = factory->uri_protocols;
         while (*protocol) {
-          gst_registry_binary_save_string (list, *protocol++);
-          pf->nuriprotocols++;
+          gst_registry_binary_save_const_string (list, *protocol++);
+          ef->nuriprotocols++;
         }
         *list = g_list_prepend (*list, subchk);
-        GST_DEBUG ("Saved %d UriTypes", pf->nuriprotocols);
+        GST_DEBUG ("Saved %d UriTypes", ef->nuriprotocols);
       } else {
         g_warning ("GStreamer feature '%s' is URI handler but does not provide"
             " any protocols it can handle", feature->name);
@@ -243,7 +305,7 @@ gst_registry_binary_save_feature (GList ** list, GstPluginFeature * feature)
 
     /* save pad-templates */
     for (walk = factory->staticpadtemplates; walk;
-        walk = g_list_next (walk), pf->npadtemplates++) {
+        walk = g_list_next (walk), ef->npadtemplates++) {
       GstStaticPadTemplate *template = walk->data;
 
       if (!gst_registry_binary_save_pad_template (list, template)) {
@@ -253,25 +315,64 @@ gst_registry_binary_save_feature (GList ** list, GstPluginFeature * feature)
     }
 
     /* pack element factory strings */
-    gst_registry_binary_save_string (list, factory->details.author);
-    gst_registry_binary_save_string (list, factory->details.description);
-    gst_registry_binary_save_string (list, factory->details.klass);
-    gst_registry_binary_save_string (list, factory->details.longname);
+    gst_registry_binary_save_const_string (list, factory->details.author);
+    gst_registry_binary_save_const_string (list, factory->details.description);
+    gst_registry_binary_save_const_string (list, factory->details.klass);
+    gst_registry_binary_save_const_string (list, factory->details.longname);
   } else if (GST_IS_TYPE_FIND_FACTORY (feature)) {
-    /* FIXME: save typefind */
+    GstBinaryTypeFindFactory *tff;
+    GstTypeFindFactory *factory = GST_TYPE_FIND_FACTORY (feature);
+    gchar *str;
+
+    /* we copy the caps here so we can simplify them before saving. This is a lot
+     * faster when loading them later on */
+    GstCaps *copy = gst_caps_copy (factory->caps);
+
+    tff = g_malloc (sizeof (GstBinaryTypeFindFactory));
+    chk =
+        gst_registry_binary_make_data (tff, sizeof (GstBinaryTypeFindFactory));
+    tff->nextensions = 0;
+    pf = (GstBinaryPluginFeature *) tff;
+
+    /* save extensions */
+    if (factory->extensions) {
+      while (factory->extensions[tff->nextensions]) {
+        gst_registry_binary_save_const_string (list,
+            factory->extensions[tff->nextensions++]);
+      }
+    }
+    /* save caps */
+    gst_caps_do_simplify (copy);
+    str = gst_caps_to_string (copy);
+    gst_caps_unref (copy);
+    gst_registry_binary_save_string (list, str);
   }
 #ifndef GST_DISABLE_INDEX
   else if (GST_IS_INDEX_FACTORY (feature)) {
-    /* FIXME: save indexers */
+    GstIndexFactory *factory = GST_INDEX_FACTORY (feature);
+
+    pf = g_malloc (sizeof (GstBinaryPluginFeature));
+    chk = gst_registry_binary_make_data (pf, sizeof (GstBinaryPluginFeature));
+    pf->rank = feature->rank;
+
+    /* pack element factory strings */
+    gst_registry_binary_save_const_string (list, factory->longdesc);
   }
 #endif
+  else {
+    GST_WARNING ("unhandled feature type '%s'", type_name);
+  }
 
-  /* pack plugin feature strings */
-  gst_registry_binary_save_string (list, feature->name);
-  gst_registry_binary_save_string (list, (gchar *) type_name);
+  if (pf) {
+    pf->rank = feature->rank;
+    *list = g_list_prepend (*list, chk);
 
-  *list = g_list_prepend (*list, chk);
-  return TRUE;
+    /* pack plugin feature strings */
+    gst_registry_binary_save_const_string (list, feature->name);
+    gst_registry_binary_save_const_string (list, (gchar *) type_name);
+
+    return TRUE;
+  }
 
   /* Errors */
 fail:
@@ -280,6 +381,7 @@ fail:
   return FALSE;
 }
 
+
 /*
  * gst_registry_binary_save_plugin:
  *
@@ -319,14 +421,14 @@ gst_registry_binary_save_plugin (GList ** list, GstRegistry * registry,
   gst_plugin_feature_list_free (plugin_features);
 
   /* pack plugin element strings */
-  gst_registry_binary_save_string (list, plugin->desc.origin);
-  gst_registry_binary_save_string (list, plugin->desc.package);
-  gst_registry_binary_save_string (list, plugin->desc.source);
-  gst_registry_binary_save_string (list, plugin->desc.license);
-  gst_registry_binary_save_string (list, plugin->desc.version);
-  gst_registry_binary_save_string (list, plugin->filename);
-  gst_registry_binary_save_string (list, plugin->desc.description);
-  gst_registry_binary_save_string (list, plugin->desc.name);
+  gst_registry_binary_save_const_string (list, plugin->desc.origin);
+  gst_registry_binary_save_const_string (list, plugin->desc.package);
+  gst_registry_binary_save_const_string (list, plugin->desc.source);
+  gst_registry_binary_save_const_string (list, plugin->desc.license);
+  gst_registry_binary_save_const_string (list, plugin->desc.version);
+  gst_registry_binary_save_const_string (list, plugin->filename);
+  gst_registry_binary_save_const_string (list, plugin->desc.description);
+  gst_registry_binary_save_const_string (list, plugin->desc.name);
 
   *list = g_list_prepend (*list, chk);
 
@@ -342,16 +444,21 @@ fail:
   return FALSE;
 }
 
+
 /**
  * gst_registry_binary_write_cache:
+ * @registry: a #GstRegistry
+ * @location: a filename
  *
- * Write the cache to file. Part of the code was taken from gstregistryxml.c
+ * Write the @registry to a cache to file at given @location.
+ * 
+ * Returns: %TRUE on success.
  */
 gboolean
 gst_registry_binary_write_cache (GstRegistry * registry, const char *location)
 {
   GList *walk;
-  char *tmp_location;
+  gchar *tmp_location;
   GstBinaryRegistryMagic *magic;
   GstBinaryChunk *magic_chunk;
   GList *to_write = NULL;
@@ -363,7 +470,7 @@ gst_registry_binary_write_cache (GstRegistry * registry, const char *location)
   tmp_location = g_strconcat (location, ".tmpXXXXXX", NULL);
   registry->cache_file = g_mkstemp (tmp_location);
   if (registry->cache_file == -1) {
-    char *dir;
+    gchar *dir;
 
     /* oops, I bet the directory doesn't exist */
     dir = g_path_get_dirname (location);
@@ -438,7 +545,7 @@ gst_registry_binary_write_cache (GstRegistry * registry, const char *location)
   g_list_free (to_write);
 
   if (close (registry->cache_file) < 0) {
-    GST_DEBUG ("Can't close registry file : %s", strerror (errno));
+    GST_DEBUG ("Can't close registry file : %s", g_strerror (errno));
     goto fail;
   }
 
@@ -459,6 +566,7 @@ fail:
   return FALSE;
 }
 
+
 /* Registry loading */
 
 /*
@@ -477,12 +585,12 @@ gst_registry_binary_check_magic (gchar ** in)
   unpack_element (*in, m, GstBinaryRegistryMagic);
 
   if (m == NULL || m->magic == NULL || m->version == NULL) {
-    GST_ERROR ("Binary registry magic structure is broken");
+    GST_WARNING ("Binary registry magic structure is broken");
     return FALSE;
   }
   if (strncmp (m->magic, GST_MAGIC_BINARY_REGISTRY_STR,
           GST_MAGIC_BINARY_REGISTRY_LEN) != 0) {
-    GST_ERROR
+    GST_WARNING
         ("Binary registry magic is different : %02x%02x%02x%02x != %02x%02x%02x%02x",
         GST_MAGIC_BINARY_REGISTRY_STR[0] & 0xff,
         GST_MAGIC_BINARY_REGISTRY_STR[1] & 0xff,
@@ -492,13 +600,14 @@ gst_registry_binary_check_magic (gchar ** in)
     return FALSE;
   }
   if (strncmp (m->version, GST_MAJORMINOR, GST_MAGIC_BINARY_VERSION_LEN)) {
-    GST_ERROR ("Binary registry magic version is different : %s != %s",
+    GST_WARNING ("Binary registry magic version is different : %s != %s",
         GST_MAJORMINOR, m->version);
     return FALSE;
   }
   return TRUE;
 }
 
+
 /*
  * gst_registry_binary_load_pad_template:
  *
@@ -516,13 +625,12 @@ gst_registry_binary_load_pad_template (GstElementFactory * factory, gchar ** in)
   GST_DEBUG ("Reading/casting for GstBinaryPadTemplate at address %p", *in);
   unpack_element (*in, pt, GstBinaryPadTemplate);
 
-
   template = g_new0 (GstStaticPadTemplate, 1);
   template->presence = pt->presence;
   template->direction = pt->direction;
 
   /* unpack pad template strings */
-  unpack_string (*in, template->name_template);
+  unpack_const_string (*in, template->name_template);
   unpack_string (*in, template->static_caps.string);
 
   __gst_element_factory_add_static_pad_template (factory, template);
@@ -531,27 +639,24 @@ gst_registry_binary_load_pad_template (GstElementFactory * factory, gchar ** in)
   return TRUE;
 }
 
+
 /*
  * gst_registry_binary_load_feature:
  *
- * Make a new GstPluginFeature from current GstBinaryPluginFeature structure
+ * Make a new GstPluginFeature from current binary plugin feature structure
  *
  * Returns: new GstPluginFeature
  */
 static gboolean
 gst_registry_binary_load_feature (GstRegistry * registry, gchar ** in,
-    gchar * plugin_name)
+    const gchar * plugin_name)
 {
-  GstBinaryPluginFeature *pf;
+  GstBinaryPluginFeature *pf = NULL;
   GstPluginFeature *feature;
   gchar *type_name = NULL, *str;
   GType type;
   guint i;
 
-  align32 (*in);
-  GST_DEBUG ("Reading/casting for GstBinaryPluginFeature at address %p", *in);
-  unpack_element (*in, pf, GstBinaryPluginFeature);
-
   /* unpack plugin feature strings */
   unpack_string (*in, type_name);
 
@@ -575,25 +680,28 @@ gst_registry_binary_load_feature (GstRegistry * registry, gchar ** in,
     goto fail;
   }
 
-  feature->rank = pf->rank;
-  feature->plugin_name = plugin_name;
-
   /* unpack more plugin feature strings */
   unpack_string (*in, feature->name);
 
   if (GST_IS_ELEMENT_FACTORY (feature)) {
+    GstBinaryElementFactory *ef;
     GstElementFactory *factory = GST_ELEMENT_FACTORY (feature);
 
+    align32 (*in);
+    GST_LOG ("Reading/casting for GstBinaryElementFactory at address %p", *in);
+    unpack_element (*in, ef, GstBinaryElementFactory);
+    pf = (GstBinaryPluginFeature *) ef;
+
     /* unpack element factory strings */
     unpack_string (*in, factory->details.longname);
     unpack_string (*in, factory->details.klass);
     unpack_string (*in, factory->details.description);
     unpack_string (*in, factory->details.author);
     GST_DEBUG ("Element factory : '%s' with npadtemplates=%d",
-        factory->details.longname, pf->npadtemplates);
+        factory->details.longname, ef->npadtemplates);
 
     /* load pad templates */
-    for (i = 0; i < pf->npadtemplates; i++) {
+    for (i = 0; i < ef->npadtemplates; i++) {
       if (!gst_registry_binary_load_pad_template (factory, in)) {
         GST_ERROR ("Error while loading binary pad template");
         goto fail;
@@ -601,36 +709,69 @@ gst_registry_binary_load_feature (GstRegistry * registry, gchar ** in,
     }
 
     /* load uritypes */
-    if (pf->nuriprotocols) {
-      GST_DEBUG ("Reading %d UriTypes at address %p", pf->nuriprotocols, *in);
+    if (ef->nuriprotocols) {
+      GST_DEBUG ("Reading %d UriTypes at address %p", ef->nuriprotocols, *in);
 
       align32 (*in);
       factory->uri_type = *((guint *) * in);
       *in += sizeof (factory->uri_type);
       //unpack_element(*in, &factory->uri_type, factory->uri_type);
 
-      factory->uri_protocols = g_new0 (gchar *, pf->nuriprotocols + 1);
-      for (i = 0; i < pf->nuriprotocols; i++) {
+      factory->uri_protocols = g_new0 (gchar *, ef->nuriprotocols + 1);
+      for (i = 0; i < ef->nuriprotocols; i++) {
         unpack_string (*in, str);
         factory->uri_protocols[i] = str;
       }
     }
     /* load interfaces */
-    GST_DEBUG ("Reading %d Interfaces at address %p", pf->ninterfaces, *in);
-    for (i = 0; i < pf->ninterfaces; i++) {
+    GST_DEBUG ("Reading %d Interfaces at address %p", ef->ninterfaces, *in);
+    for (i = 0; i < ef->ninterfaces; i++) {
       unpack_string (*in, str);
       __gst_element_factory_add_interface (factory, str);
       g_free (str);
     }
   } else if (GST_IS_TYPE_FIND_FACTORY (feature)) {
-    /* FIXME: load typefind */
+    GstBinaryTypeFindFactory *tff;
+    GstTypeFindFactory *factory = GST_TYPE_FIND_FACTORY (feature);
+
+    align32 (*in);
+    GST_DEBUG ("Reading/casting for GstBinaryPluginFeature at address %p", *in);
+    unpack_element (*in, tff, GstBinaryTypeFindFactory);
+    pf = (GstBinaryPluginFeature *) tff;
+
+    /* load caps */
+    unpack_string (*in, str);
+    factory->caps = gst_caps_from_string (str);
+    g_free (str);
+    /* load extensions */
+    if (tff->nextensions) {
+      GST_DEBUG ("Reading %d Typefind extensions at address %p",
+          tff->nextensions, *in);
+      factory->extensions = g_new0 (gchar *, tff->nextensions + 1);
+      for (i = 0; i < tff->nextensions; i++) {
+        unpack_string (*in, str);
+        factory->extensions[i] = str;
+      }
+    }
   }
 #ifndef GST_DISABLE_INDEX
   else if (GST_IS_INDEX_FACTORY (feature)) {
-    /* FIXME: load indexers */
+    GstIndexFactory *factory = GST_INDEX_FACTORY (feature);
+
+    align32 (*in);
+    GST_DEBUG ("Reading/casting for GstBinaryPluginFeature at address %p", *in);
+    unpack_element (*in, pf, GstBinaryPluginFeature);
+
+    /* unpack index factory strings */
+    unpack_string (*in, factory->longdesc);
   }
 #endif
 
+  feature->rank = pf->rank;
+
+  /* should already be the interned string, but better make sure */
+  feature->plugin_name = g_intern_string (plugin_name);
+
   gst_registry_add_feature (registry, feature);
   GST_DEBUG ("Added feature %s", feature->name);
 
@@ -647,6 +788,7 @@ fail:
   return FALSE;
 }
 
+
 /*
  * gst_registry_binary_load_plugin:
  *
@@ -662,7 +804,7 @@ gst_registry_binary_load_plugin (GstRegistry * registry, gchar ** in)
   guint i;
 
   align32 (*in);
-  GST_DEBUG ("Reading/casting for GstBinaryPluginElement at address %p", *in);
+  GST_LOG ("Reading/casting for GstBinaryPluginElement at address %p", *in);
   unpack_element (*in, pe, GstBinaryPluginElement);
 
   if (pe->nfeatures < 0) {
@@ -675,33 +817,30 @@ gst_registry_binary_load_plugin (GstRegistry * registry, gchar ** in)
     return FALSE;
   }
 
-  GST_DEBUG ("Adding plugin with %d features from binary registry",
-      pe->nfeatures);
-
   plugin = g_object_new (GST_TYPE_PLUGIN, NULL);
 
+  /* TODO: also set GST_PLUGIN_FLAG_CONST */
   plugin->flags |= GST_PLUGIN_FLAG_CACHED;
   plugin->file_mtime = pe->file_mtime;
   plugin->file_size = pe->file_size;
 
   /* unpack plugin element strings */
-  unpack_string (*in, plugin->desc.name);
+  unpack_const_string (*in, plugin->desc.name);
   unpack_string (*in, plugin->desc.description);
   unpack_string (*in, plugin->filename);
-  unpack_string (*in, plugin->desc.version);
-  unpack_string (*in, plugin->desc.license);
-  unpack_string (*in, plugin->desc.source);
-  unpack_string (*in, plugin->desc.package);
-  unpack_string (*in, plugin->desc.origin);
-  GST_DEBUG ("read strings for '%s'", plugin->desc.name);
+  unpack_const_string (*in, plugin->desc.version);
+  unpack_const_string (*in, plugin->desc.license);
+  unpack_const_string (*in, plugin->desc.source);
+  unpack_const_string (*in, plugin->desc.package);
+  unpack_const_string (*in, plugin->desc.origin);
+  GST_LOG ("read strings for '%s'", plugin->desc.name);
 
   plugin->basename = g_path_get_basename (plugin->filename);
   gst_registry_add_plugin (registry, plugin);
-  GST_DEBUG ("Added plugin '%s' from binary registry", plugin->desc.name);
-  GST_DEBUG ("Number of features %d", pe->nfeatures);
+  GST_INFO ("Added plugin '%s' plugin with %d features from binary registry",
+      plugin->desc.name, pe->nfeatures);
   for (i = 0; i < pe->nfeatures; i++) {
-    if (!gst_registry_binary_load_feature (registry, in,
-            g_strdup (plugin->desc.name))) {
+    if (!gst_registry_binary_load_feature (registry, in, plugin->desc.name)) {
       GST_ERROR ("Error while loading binary feature");
       goto fail;
     }
@@ -735,6 +874,7 @@ gst_registry_binary_read_cache (GstRegistry * registry, const char *location)
   gdouble seconds;
   gsize size;
   GError *err = NULL;
+  gboolean res = FALSE;
 
   /* make sure these types exist */
   GST_TYPE_ELEMENT_FACTORY;
@@ -747,30 +887,38 @@ gst_registry_binary_read_cache (GstRegistry * registry, const char *location)
 
   mapped = g_mapped_file_new (location, FALSE, &err);
   if (err != NULL) {
-    GST_INFO ("Unable to mmap file: %s\n", err->message);
+    GST_INFO ("Unable to mmap file %s : %s", location, err->message);
     g_error_free (err);
-    return FALSE;
-  }
+    err = NULL;
 
-  if ((contents = g_mapped_file_get_contents (mapped)) == NULL) {
-    GST_ERROR ("Can't load file %s : %s", location, strerror (errno));
-    g_mapped_file_free (mapped);
-    return FALSE;
+    g_file_get_contents (location, &contents, &size, &err);
+    if (err != NULL) {
+      GST_INFO ("Unable to read file %s : %s", location, err->message);
+      g_timer_destroy (timer);
+      g_error_free (err);
+      return FALSE;
+    }
+  } else {
+    if ((contents = g_mapped_file_get_contents (mapped)) == NULL) {
+      GST_ERROR ("Can't load file %s : %s", location, g_strerror (errno));
+      goto Error;
+    }
+    /* check length for header */
+    size = g_mapped_file_get_length (mapped);
   }
   /* in is a cursor pointer, we initialize it with the begin of registry and is updated on each read */
   in = contents;
-  GST_DEBUG ("File mapped at address %p", in);
-  /* check length for header */
-  size = g_mapped_file_get_length (mapped);
+  GST_DEBUG ("File data at address %p", in);
   if (size < sizeof (GstBinaryRegistryMagic)) {
-    GST_INFO ("No or broken registry header");
-    return FALSE;
+    GST_ERROR ("No or broken registry header");
+    goto Error;
   }
   /* check if header is valid */
   if (!gst_registry_binary_check_magic (&in)) {
-    GST_ERROR ("Binary registry type not recognized (invalid magic)");
-    g_mapped_file_free (mapped);
-    return FALSE;
+    GST_ERROR
+        ("Binary registry type not recognized (invalid magic) for file at %s",
+        location);
+    goto Error;
   }
 
   /* check if there are plugins in the file */
@@ -778,27 +926,35 @@ gst_registry_binary_read_cache (GstRegistry * registry, const char *location)
   if (!(((size_t) in + sizeof (GstBinaryPluginElement)) <
           (size_t) contents + size)) {
     GST_INFO ("No binary plugins structure to read");
-    return TRUE;                /* empty file, this is not an error */
-  }
-
-  for (;
-      ((size_t) in + sizeof (GstBinaryPluginElement)) <
-      (size_t) contents + size;) {
-    GST_INFO ("reading binary registry %" G_GSIZE_FORMAT "(%x)/%"
-        G_GSIZE_FORMAT, (size_t) in - (size_t) contents,
-        (guint) ((size_t) in - (size_t) contents), size);
-    if (!gst_registry_binary_load_plugin (registry, &in)) {
-      GST_ERROR ("Problem while reading binary registry");
-      return FALSE;
+    /* empty file, this is not an error */
+  } else {
+    for (;
+        ((size_t) in + sizeof (GstBinaryPluginElement)) <
+        (size_t) contents + size;) {
+      GST_DEBUG ("reading binary registry %" G_GSIZE_FORMAT "(%x)/%"
+          G_GSIZE_FORMAT, (size_t) in - (size_t) contents,
+          (guint) ((size_t) in - (size_t) contents), size);
+      if (!gst_registry_binary_load_plugin (registry, &in)) {
+        GST_ERROR ("Problem while reading binary registry");
+        goto Error;
+      }
     }
   }
 
   g_timer_stop (timer);
   seconds = g_timer_elapsed (timer, NULL);
-  g_timer_destroy (timer);
 
   GST_INFO ("loaded %s in %lf seconds", location, seconds);
 
-  g_mapped_file_free (mapped);
-  return TRUE;
+  res = TRUE;
+  /* TODO: once we re-use the pointers to registry contents return here */
+
+Error:
+  g_timer_destroy (timer);
+  if (mapped) {
+    g_mapped_file_free (mapped);
+  } else {
+    g_free (contents);
+  }
+  return res;
 }
