@@ -27,7 +27,8 @@
  * A #GstStructure is a collection of key/value pairs. The keys are expressed
  * as GQuarks and the values can be of any GType.
  *
- * In addition to the key/value pairs, a #GstStructure also has a name.
+ * In addition to the key/value pairs, a #GstStructure also has a name. The name
+ * starts with a letter and can be folled by letters, numbers and any of "/-_.:".
  * 
  * #GstStructure is used by various GStreamer subsystems to store information
  * in a flexible and extensible way. A #GstStructure does not have a refcount
@@ -48,7 +49,7 @@
  * Fields can be removed with gst_structure_remove_field() or
  * gst_structure_remove_fields().
  *
- * Last reviewed on 2005-11-09 (0.9.4)
+ * Last reviewed on 2007-10-16 (0.10.15)
  */
 
 #ifdef HAVE_CONFIG_H
@@ -137,18 +138,47 @@ gst_structure_id_empty_new (GQuark quark)
   return gst_structure_id_empty_new_with_size (quark, 0);
 }
 
+static gboolean
+gst_structure_validate_name (const gchar * name)
+{
+  const gchar *s;
+
+  g_return_val_if_fail (name != NULL, FALSE);
+
+  if (!g_ascii_isalpha (*name)) {
+    GST_WARNING ("Invalid character '%c' at offset 0 in structure name: %s",
+        *name, name);
+    return FALSE;
+  }
+
+  /* FIXME 0.11: don't allow spaces */
+  /* FIXME: test name string more */
+  s = &name[1];
+  while (*s && (g_ascii_isalnum (*s) || strchr ("/-_.:+ ", *s) != NULL))
+    s++;
+  if (*s != '\0') {
+    GST_WARNING ("Invalid character '%c' at offset %lu in structure name: %s",
+        *s, ((gulong) s - (gulong) name), name);
+    return FALSE;
+  }
+
+  return TRUE;
+}
+
 /**
  * gst_structure_empty_new:
  * @name: name of new structure
  *
- * Creates a new, empty #GstStructure with the given name.
+ * Creates a new, empty #GstStructure with the given @name.
+ *
+ * See gst_structure_set_name() for constraints on the @name parameter.
  *
  * Returns: a new, empty #GstStructure
  */
 GstStructure *
 gst_structure_empty_new (const gchar * name)
 {
-  g_return_val_if_fail (name != NULL, NULL);
+  g_return_val_if_fail (gst_structure_validate_name (name), NULL);
 
   return gst_structure_id_empty_new_with_size (g_quark_from_string (name), 0);
 }
@@ -189,9 +219,11 @@ gst_structure_new (const gchar * name, const gchar * firstfield, ...)
  * @firstfield: name of first field to set
  * @varargs: variable argument list
  *
- * Creates a new #GstStructure with the given name.  Structure fields
+ * Creates a new #GstStructure with the given @name.  Structure fields
  * are set according to the varargs in a manner similar to
- * @gst_structure_new.
+ * gst_structure_new().
+ *
+ * See gst_structure_set_name() for constraints on the @name parameter.
  *
  * Returns: a new #GstStructure
  */
@@ -331,6 +363,9 @@ gst_structure_has_name (const GstStructure * structure, const gchar * name)
   g_return_val_if_fail (structure != NULL, FALSE);
   g_return_val_if_fail (name != NULL, FALSE);
 
+  /* getting the string is cheap and comparing short strings is too
+   * should be faster than getting the quark for name and comparing the quarks
+   */
   structure_name = g_quark_to_string (structure->name);
 
   return (structure_name && strcmp (structure_name, name) == 0);
@@ -357,15 +392,16 @@ gst_structure_get_name_id (const GstStructure * structure)
  * @structure: a #GstStructure
  * @name: the new name of the structure
  *
- * Sets the name of the structure to the given name.  The string
- * provided is copied before being used.
+ * Sets the name of the structure to the given @name.  The string
+ * provided is copied before being used. It must not be empty, start with a
+ * letter and can be followed by letters, numbers and any of "/-_.:".
  */
 void
 gst_structure_set_name (GstStructure * structure, const gchar * name)
 {
   g_return_if_fail (structure != NULL);
-  g_return_if_fail (name != NULL);
   g_return_if_fail (IS_MUTABLE (structure));
+  g_return_if_fail (gst_structure_validate_name (name));
 
   structure->name = g_quark_from_string (name);
 }
@@ -1042,6 +1078,44 @@ gst_structure_get_int (const GstStructure * structure,
   return TRUE;
 }
 
+/**
+ * gst_structure_get_uint:
+ * @structure: a #GstStructure
+ * @fieldname: the name of a field
+ * @value: a pointer to a uint to set
+ *
+ * Sets the uint pointed to by @value corresponding to the value of the
+ * given field.  Caller is responsible for making sure the field exists
+ * and has the correct type.
+ *
+ * Returns: %TRUE if the value could be set correctly. If there was no field
+ * with @fieldname or the existing field did not contain a uint, this function
+ * returns %FALSE.
+ *
+ * Since: 0.10.15
+ */
+gboolean
+gst_structure_get_uint (const GstStructure * structure,
+    const gchar * fieldname, guint * value)
+{
+  GstStructureField *field;
+
+  g_return_val_if_fail (structure != NULL, FALSE);
+  g_return_val_if_fail (fieldname != NULL, FALSE);
+  g_return_val_if_fail (value != NULL, FALSE);
+
+  field = gst_structure_get_field (structure, fieldname);
+
+  if (field == NULL)
+    return FALSE;
+  if (!G_VALUE_HOLDS_UINT (&field->value))
+    return FALSE;
+
+  *value = g_value_get_uint (&field->value);
+
+  return TRUE;
+}
+
 /**
  * gst_structure_get_fourcc:
  * @structure: a #GstStructure
@@ -1348,6 +1422,8 @@ gst_structure_get_abbrs (gint * n_abbrs)
       {"str", G_TYPE_STRING}
       ,
       {"s", G_TYPE_STRING}
+      ,
+      {"structure", GST_TYPE_STRUCTURE}
     };
     num = G_N_ELEMENTS (dyn_abbrs);
     /* permanently allocate and copy the array now */
@@ -1426,6 +1502,7 @@ gst_structure_value_get_generic_type (GValue * val)
   return G_VALUE_TYPE (val);
 }
 
+/* keep in sync with gstvalue.c */
 #define GST_ASCII_IS_STRING(c) (g_ascii_isalnum((c)) || ((c) == '_') || \
       ((c) == '-') || ((c) == '+') || ((c) == '/') || ((c) == ':') || \
       ((c) == '.'))
@@ -1455,7 +1532,6 @@ gst_structure_to_string (const GstStructure * structure)
   g_return_val_if_fail (structure != NULL, NULL);
 
   s = g_string_new ("");
-  /* FIXME this string may need to be escaped */
   g_string_append_printf (s, "%s", g_quark_to_string (structure->name));
   for (i = 0; i < structure->fields->len; i++) {
     char *t;
@@ -1466,10 +1542,14 @@ gst_structure_to_string (const GstStructure * structure)
     t = gst_value_serialize (&field->value);
     type = gst_structure_value_get_generic_type (&field->value);
 
+    /* FIXME: do we need to escape fieldnames? */
     g_string_append_printf (s, ", %s=(%s)%s", g_quark_to_string (field->name),
         gst_structure_to_abbr (type), GST_STR_NULL (t));
     g_free (t);
   }
+
+  g_string_append_c (s, ';');
+
   return g_string_free (s, FALSE);
 }
 
@@ -1528,7 +1608,6 @@ gst_structure_parse_range (gchar * s, gchar ** after, GValue * value,
   GType range_type;
   gboolean ret;
 
-
   if (*s != '[')
     return FALSE;
   s++;
@@ -1822,23 +1901,39 @@ gst_structure_from_string (const gchar * string, gchar ** end)
   copy = g_strdup (string);
   r = copy;
 
-  name = r;
-  if (!gst_structure_parse_string (r, &w, &r))
-    goto error;
-
-  while (g_ascii_isspace (*r) || (r[0] == '\\' && g_ascii_isspace (r[1])))
+  /* skipe spaces */
+  while (*r && (g_ascii_isspace (*r) || (r[0] == '\\'
+              && g_ascii_isspace (r[1]))))
     r++;
-  if (*r != 0 && *r != ';' && *r != ',')
+
+  name = r;
+  if (!gst_structure_parse_string (r, &w, &r)) {
+    GST_WARNING ("Failed to parse structure string");
     goto error;
+  }
 
   save = *w;
   *w = 0;
   structure = gst_structure_empty_new (name);
   *w = save;
 
-  while (*r && (*r != ';')) {
-    if (*r != ',')
+  do {
+    while (*r && (g_ascii_isspace (*r) || (r[0] == '\\'
+                && g_ascii_isspace (r[1]))))
+      r++;
+    if (*r == ';') {
+      /* end of structure, get the next char and finish */
+      r++;
+      break;
+    }
+    if (*r == '\0') {
+      /* accept \0 as end delimiter */
+      break;
+    }
+    if (*r != ',') {
+      GST_WARNING ("Failed to find delimiter, r=%s", r);
       goto error;
+    }
     r++;
     while (*r && (g_ascii_isspace (*r) || (r[0] == '\\'
                 && g_ascii_isspace (r[1]))))
@@ -1848,10 +1943,8 @@ gst_structure_from_string (const gchar * string, gchar ** end)
     if (!gst_structure_parse_field (r, &r, &field))
       goto error;
     gst_structure_set_field (structure, &field);
-    while (*r && (g_ascii_isspace (*r) || (r[0] == '\\'
-                && g_ascii_isspace (r[1]))))
-      r++;
-  }
+
+  } while (TRUE);
 
   if (end)
     *end = (char *) string + (r - copy);
@@ -2129,23 +2222,23 @@ gst_structure_fixate_field_nearest_fraction (GstStructure * structure,
 
     gst_value_set_fraction (&target, target_numerator, target_denominator);
 
+    best = NULL;
+
     n = gst_value_list_get_size (value);
     for (i = 0; i < n; i++) {
       list_value = gst_value_list_get_value (value, i);
       if (G_VALUE_TYPE (list_value) == GST_TYPE_FRACTION) {
-        if (best == NULL) {
-          best = list_value;
-          gst_value_set_fraction (&best_diff, 0, 1);
-        } else {
-          if (gst_value_compare (list_value, &target) == GST_VALUE_LESS_THAN)
-            gst_value_fraction_subtract (&cur_diff, &target, list_value);
-          else
-            gst_value_fraction_subtract (&cur_diff, list_value, &target);
 
-          if (gst_value_compare (&cur_diff, &best_diff) == GST_VALUE_LESS_THAN) {
-            best = list_value;
-            g_value_copy (&cur_diff, &best_diff);
-          }
+        if (gst_value_compare (list_value, &target) == GST_VALUE_LESS_THAN)
+          gst_value_fraction_subtract (&cur_diff, &target, list_value);
+        else
+          gst_value_fraction_subtract (&cur_diff, list_value, &target);
+
+        if (!best
+            || gst_value_compare (&cur_diff,
+                &best_diff) == GST_VALUE_LESS_THAN) {
+          best = list_value;
+          g_value_copy (&cur_diff, &best_diff);
         }
       }
     }
@@ -2158,6 +2251,5 @@ gst_structure_fixate_field_nearest_fraction (GstStructure * structure,
     g_value_unset (&target);
     return res;
   }
-
   return FALSE;
 }
