@@ -22,6 +22,7 @@
 
 #include <gst/gstconfig.h>
 #include <gst/gstcaps.h>
+#include <gst/gststructure.h>
 
 G_BEGIN_DECLS
 
@@ -140,6 +141,14 @@ G_BEGIN_DECLS
  */
 #define GST_VALUE_HOLDS_CAPS(x)		(G_VALUE_HOLDS(x, GST_TYPE_CAPS))
 
+/**
+ * GST_VALUE_HOLDS_STRUCTURE:
+ * @x: the #GValue to check
+ *
+ * Checks if the given #GValue contains a #GST_TYPE_STRUCTURE value.
+ */
+#define GST_VALUE_HOLDS_STRUCTURE(x)		(G_VALUE_HOLDS(x, GST_TYPE_STRUCTURE))
+
 /**
  * GST_VALUE_HOLDS_BUFFER:
  * @x: the #GValue to check
@@ -449,6 +458,12 @@ G_CONST_RETURN GstCaps *
 void		gst_value_set_caps		(GValue		*value,
 						 const GstCaps  *caps);
 
+/* structure */
+G_CONST_RETURN GstStructure *
+		gst_value_get_structure		(const GValue	*value);
+void		gst_value_set_structure		(GValue		*value,
+						 const GstStructure  *structure);
+
 /* fraction */
 void		gst_value_set_fraction		(GValue		*value,
 						 gint		numerator,
@@ -518,9 +533,6 @@ void		gst_value_register_subtract_func (GType		minuend_type,
 /* fixation */
 gboolean	gst_value_is_fixed		(const GValue   *value);
 
-/* private */
-void		_gst_value_initialize		(void);
-
 G_END_DECLS
 
 #endif
