@@ -81,10 +81,10 @@
  * to the #GstBaseSrc::create function.
  * </para>
  * <para>
- * #GstBaseSrc has support for live sources. Live sources are sources that 
- * produce data at a fixed rate, such as audio or video capture devices. A 
- * typical live source also provides a clock to publish the rate at which 
- * they produce data.
+ * #GstBaseSrc has support for live sources. Live sources are sources that when
+ * paused discard data, such as audio or video capture devices. A typical live
+ * source also produces data at a fixed rate and thus provides a clock to publish
+ * this rate.
  * Use gst_base_src_set_live() to activate the live source mode.
  * </para>
  * <para>
@@ -95,21 +95,32 @@
  * </para>
  * <para>
  * A typical live source will timestamp the buffers it creates with the 
- * current stream time of the pipeline. This is one reason why a live source
+ * current running time of the pipeline. This is one reason why a live source
  * can only produce data in the PLAYING state, when the clock is actually 
- * distributed and running.
+ * distributed and running. 
  * </para>
  * <para>
- * Live sources that synchronize and block on the clock (and audio source, for
+ * Live sources that synchronize and block on the clock (an audio source, for
  * example) can since 0.10.12 use gst_base_src_wait_playing() when the ::create
  * function was interrupted by a state change to PAUSED.
  * </para>
  * <para>
  * The #GstBaseSrc::get_times method can be used to implement pseudo-live 
- * sources. The base source will wait for the specified stream time returned in 
- * #GstBaseSrc::get_times before pushing out the buffer. 
+ * sources.
  * It only makes sense to implement the ::get_times function if the source is 
- * a live source.
+ * a live source. The ::get_times function should return timestamps starting
+ * from 0, as if it were a non-live source. The base class will make sure that
+ * the timestamps are transformed into the current running_time.
+ * The base source will then wait for the calculated running_time before pushing
+ * out the buffer.
+ * </para>
+ * <para>
+ * For live sources, the base class will by default report a latency of 0.
+ * For pseudo live sources, the base class will by default measure the difference
+ * between the first buffer timestamp and the start time of get_times and will
+ * report this value as the latency. 
+ * Subclasses should override the query function when this behaviour is not
+ * acceptable.
  * </para>
  * <para>
  * There is only support in #GstBaseSrc for exactly one source pad, which 
@@ -172,7 +183,7 @@
  * thread might be blocked in PREROLL.
  * </para>
  * <para>
- * Last reviewed on 2006-09-27 (0.10.11)
+ * Last reviewed on 2007-09-13 (0.10.15)
  * </para>
  * </refsect2>
  */
@@ -213,6 +224,7 @@ enum
 #define DEFAULT_BLOCKSIZE       4096
 #define DEFAULT_NUM_BUFFERS     -1
 #define DEFAULT_TYPEFIND	FALSE
+#define DEFAULT_DO_TIMESTAMP	FALSE
 
 enum
 {
@@ -220,6 +232,7 @@ enum
   PROP_BLOCKSIZE,
   PROP_NUM_BUFFERS,
   PROP_TYPEFIND,
+  PROP_DO_TIMESTAMP
 };
 
 #define GST_BASE_SRC_GET_PRIVATE(obj)  \
@@ -230,10 +243,21 @@ struct _GstBaseSrcPrivate
   gboolean last_sent_eos;       /* last thing we did was send an EOS (we set this
                                  * to avoid the sending of two EOS in some cases) */
   gboolean discont;
+  gboolean flushing;
 
   /* two segments to be sent in the streaming thread with STREAM_LOCK */
   GstEvent *close_segment;
   GstEvent *start_segment;
+
+  /* startup latency is the time it takes between going to PLAYING and producing
+   * the first BUFFER with running_time 0. This value is included in the latency
+   * reporting. */
+  GstClockTime latency;
+  /* timestamp offset, this is the offset add to the values of gst_times for
+   * pseudo live sources */
+  GstClockTimeDiff ts_offset;
+
+  gboolean do_timestamp;
 };
 
 static GstElementClass *parent_class = NULL;
@@ -291,8 +315,8 @@ static gboolean gst_base_src_default_query (GstBaseSrc * src, GstQuery * query);
 static gboolean gst_base_src_default_prepare_seek_segment (GstBaseSrc * src,
     GstEvent * event, GstSegment * segment);
 
-static gboolean gst_base_src_unlock (GstBaseSrc * basesrc);
-static gboolean gst_base_src_unlock_stop (GstBaseSrc * basesrc);
+static gboolean gst_base_src_set_flushing (GstBaseSrc * basesrc,
+    gboolean flushing, gboolean live_play, gboolean unlock, gboolean * playing);
 static gboolean gst_base_src_start (GstBaseSrc * basesrc);
 static gboolean gst_base_src_stop (GstBaseSrc * basesrc);
 
@@ -342,6 +366,10 @@ gst_base_src_class_init (GstBaseSrcClass * klass)
       g_param_spec_boolean ("typefind", "Typefind",
           "Run typefind before negotiating", DEFAULT_TYPEFIND,
           G_PARAM_READWRITE));
+  g_object_class_install_property (gobject_class, PROP_DO_TIMESTAMP,
+      g_param_spec_boolean ("do-timestamp", "Do timestamp",
+          "Apply current stream time to buffers", DEFAULT_DO_TIMESTAMP,
+          G_PARAM_READWRITE));
 
   gstelement_class->change_state =
       GST_DEBUG_FUNCPTR (gst_base_src_change_state);
@@ -410,6 +438,7 @@ gst_base_src_init (GstBaseSrc * basesrc, gpointer g_class)
   /* we operate in BYTES by default */
   gst_base_src_set_format (basesrc, GST_FORMAT_BYTES);
   basesrc->data.ABI.typefind = DEFAULT_TYPEFIND;
+  basesrc->priv->do_timestamp = DEFAULT_DO_TIMESTAMP;
 
   GST_OBJECT_FLAG_UNSET (basesrc, GST_BASE_SRC_STARTED);
 
@@ -428,7 +457,7 @@ gst_base_src_finalize (GObject * object)
   g_cond_free (basesrc->live_cond);
 
   event_p = &basesrc->data.ABI.pending_seek;
-  gst_event_replace ((GstEvent **) event_p, NULL);
+  gst_event_replace (event_p, NULL);
 
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
@@ -455,7 +484,6 @@ GstFlowReturn
 gst_base_src_wait_playing (GstBaseSrc * src)
 {
   /* block until the state changes, or we get a flush, or something */
-  GST_LIVE_LOCK (src);
   if (src->is_live) {
     while (G_UNLIKELY (!src->live_running)) {
       GST_DEBUG ("live source signal waiting");
@@ -464,23 +492,16 @@ gst_base_src_wait_playing (GstBaseSrc * src)
       GST_LIVE_WAIT (src);
       GST_DEBUG ("live source unlocked");
     }
-    /* FIXME, use another variable to signal stopping so that we don't
-     * have to grab another lock. */
-    GST_OBJECT_LOCK (src->srcpad);
-    if (G_UNLIKELY (GST_PAD_IS_FLUSHING (src->srcpad)))
+    if (src->priv->flushing)
       goto flushing;
-    GST_OBJECT_UNLOCK (src->srcpad);
   }
-  GST_LIVE_UNLOCK (src);
 
   return GST_FLOW_OK;
 
   /* ERRORS */
 flushing:
   {
-    GST_DEBUG_OBJECT (src, "pad is flushing");
-    GST_OBJECT_UNLOCK (src->srcpad);
-    GST_LIVE_UNLOCK (src);
+    GST_DEBUG_OBJECT (src, "we are flushing");
     return GST_FLOW_WRONG_STATE;
   }
 }
@@ -502,9 +523,9 @@ flushing:
 void
 gst_base_src_set_live (GstBaseSrc * src, gboolean live)
 {
-  GST_LIVE_LOCK (src);
+  GST_OBJECT_LOCK (src);
   src->is_live = live;
-  GST_LIVE_UNLOCK (src);
+  GST_OBJECT_UNLOCK (src);
 }
 
 /**
@@ -520,9 +541,9 @@ gst_base_src_is_live (GstBaseSrc * src)
 {
   gboolean result;
 
-  GST_LIVE_LOCK (src);
+  GST_OBJECT_LOCK (src);
   result = src->is_live;
-  GST_LIVE_UNLOCK (src);
+  GST_OBJECT_UNLOCK (src);
 
   return result;
 }
@@ -538,7 +559,7 @@ gst_base_src_is_live (GstBaseSrc * src)
  * If a format of GST_FORMAT_BYTES is set, the element will be able to
  * operate in pull mode if the #GstBaseSrc::is_seekable returns TRUE.
  *
- * @Since: 0.10.1
+ * Since: 0.10.1
  */
 void
 gst_base_src_set_format (GstBaseSrc * src, GstFormat format)
@@ -554,8 +575,8 @@ gst_base_src_set_format (GstBaseSrc * src, GstFormat format)
  * @max_latency: the max latency of the source
  *
  * Query the source for the latency parameters. @live will be TRUE when @src is
- * configured as a live source. @min_latency will be set as the latency between
- * calling the create function and the timestamp on the resulting buffer.
+ * configured as a live source. @min_latency will be set to the difference
+ * between the running time and the timestamp of the first buffer.
  * @max_latency is always the undefined value of -1.
  *
  * This function is mostly used by subclasses. 
@@ -568,18 +589,74 @@ gboolean
 gst_base_src_query_latency (GstBaseSrc * src, gboolean * live,
     GstClockTime * min_latency, GstClockTime * max_latency)
 {
-  GST_LIVE_LOCK (src);
+  GstClockTime min;
+
+  GST_OBJECT_LOCK (src);
   if (live)
     *live = src->is_live;
+
+  /* if we have a startup latency, report this one, else report 0. Subclasses
+   * are supposed to override the query function if they want something
+   * else. */
+  if (src->priv->latency != -1)
+    min = src->priv->latency;
+  else
+    min = 0;
+
   if (min_latency)
-    *min_latency = 0;
+    *min_latency = min;
   if (max_latency)
     *max_latency = -1;
-  GST_LIVE_UNLOCK (src);
+
+  GST_LOG_OBJECT (src, "latency: live %d, min %" GST_TIME_FORMAT
+      ", max %" GST_TIME_FORMAT, src->is_live, GST_TIME_ARGS (min),
+      GST_TIME_ARGS (-1));
+  GST_OBJECT_UNLOCK (src);
 
   return TRUE;
 }
 
+/**
+ * gst_base_src_set_do_timestamp:
+ * @src: the source
+ * @timestamp: enable or disable timestamping
+ *
+ * Configure @src to automatically timestamp outgoing buffers based on the
+ * current running_time of the pipeline. This property is mostly useful for live
+ * sources.
+ *
+ * Since: 0.10.15
+ */
+void
+gst_base_src_set_do_timestamp (GstBaseSrc * src, gboolean timestamp)
+{
+  GST_OBJECT_LOCK (src);
+  src->priv->do_timestamp = timestamp;
+  GST_OBJECT_UNLOCK (src);
+}
+
+/**
+ * gst_base_src_get_do_timestamp:
+ * @src: the source
+ *
+ * Query if @src timestamps outgoing buffers based on the current running_time.
+ *
+ * Returns: %TRUE if the base class will automatically timestamp outgoing buffers.
+ *
+ * Since: 0.10.15
+ */
+gboolean
+gst_base_src_get_do_timestamp (GstBaseSrc * src)
+{
+  gboolean res;
+
+  GST_OBJECT_LOCK (src);
+  res = src->priv->do_timestamp;
+  GST_OBJECT_UNLOCK (src);
+
+  return res;
+}
+
 static gboolean
 gst_base_src_setcaps (GstPad * pad, GstCaps * caps)
 {
@@ -707,14 +784,20 @@ gst_base_src_default_query (GstBaseSrc * src, GstQuery * query)
         {
           gint64 duration;
 
+          /* this is the duration as configured by the subclass. */
           duration = src->segment.duration;
 
           if (duration != -1) {
-            /* convert to requested format */
+            /* convert to requested format, if this fails, we have a duration
+             * but we cannot answer the query, we must return FALSE. */
             res =
                 gst_pad_query_convert (src->srcpad, src->segment.format,
                 duration, &format, &duration);
           } else {
+            /* The subclass did not configure a duration, we assume that the
+             * media has an unknown duration then and we return TRUE to report
+             * this. Note that this is not the same as returning FALSE, which
+             * means that we cannot report the duration at all. */
             res = TRUE;
           }
           gst_query_set_duration (query, format, duration);
@@ -784,6 +867,10 @@ gst_base_src_default_query (GstBaseSrc * src, GstQuery * query)
       /* Subclasses should override and implement something usefull */
       res = gst_base_src_query_latency (src, &live, &min, &max);
 
+      GST_LOG_OBJECT (src, "report latency: live %d, min %" GST_TIME_FORMAT
+          ", max %" GST_TIME_FORMAT, live, GST_TIME_ARGS (min),
+          GST_TIME_ARGS (max));
+
       gst_query_set_latency (query, live, min, max);
       break;
     }
@@ -944,7 +1031,10 @@ gst_base_src_prepare_seek_segment (GstBaseSrc * src, GstEvent * event,
  * acquire the STREAM_LOCK which is taken when we are in the
  * _loop() function or when a getrange() is called. Normally
  * we will not receive a seek if we are operating in pull mode
- * though.
+ * though. When we operate as a live source we might block on the live
+ * cond, which does not release the STREAM_LOCK. Therefore we will try
+ * to grab the LIVE_LOCK instead of the STREAM_LOCK to make sure it is
+ * safe to perform the seek.
  *
  * When we are in the loop() function, we might be in the middle
  * of pushing a buffer, which might block in a sink. To make sure
@@ -957,10 +1047,11 @@ gst_base_src_prepare_seek_segment (GstBaseSrc * src, GstEvent * event,
  * blocks on the sample we might wait a very long time until the sink
  * unblocks the sample. In any case we acquire the STREAM_LOCK and
  * can continue the seek. A non-flushing seek is normally done in a 
- * running pipeline to perform seamless playback.
+ * running pipeline to perform seamless playback, this means that the sink is
+ * PLAYING and will return from its chain function.
  * In the case of a non-flushing seek we need to make sure that the
  * data we output after the seek is continuous with the previous data,
- * this is because a non-flushing seek does not reset the stream-time
+ * this is because a non-flushing seek does not reset the running-time
  * to 0. We do this by closing the currently running segment, ie. sending
  * a new_segment event with the stop position set to the last processed 
  * position.
@@ -990,7 +1081,7 @@ gst_base_src_perform_seek (GstBaseSrc * src, GstEvent * event, gboolean unlock)
   GstSeekFlags flags;
   GstSeekType cur_type, stop_type;
   gint64 cur, stop;
-  gboolean flush;
+  gboolean flush, playing;
   gboolean update;
   gboolean relative_seek = FALSE;
   gboolean seekseg_configured = FALSE;
@@ -1030,17 +1121,15 @@ gst_base_src_perform_seek (GstBaseSrc * src, GstEvent * event, gboolean unlock)
   else
     gst_pad_pause_task (src->srcpad);
 
-  /* unblock streaming thread */
-  if (unlock)
-    gst_base_src_unlock (src);
+  /* unblock streaming thread. */
+  gst_base_src_set_flushing (src, TRUE, FALSE, unlock, &playing);
 
   /* grab streaming lock, this should eventually be possible, either
-   * because the task is paused or our streaming thread stopped 
-   * because our peer is flushing. */
+   * because the task is paused, our streaming thread stopped 
+   * or because our peer is flushing. */
   GST_PAD_STREAM_LOCK (src->srcpad);
 
-  if (unlock)
-    gst_base_src_unlock_stop (src);
+  gst_base_src_set_flushing (src, FALSE, playing, unlock, NULL);
 
   /* If we configured the seeksegment above, don't overwrite it now. Otherwise
    * copy the current segment info into the temp segment that we can actually
@@ -1174,7 +1263,8 @@ gst_base_src_get_query_types (GstElement * element)
   return query_types;
 }
 
-/* all events send to this element directly
+/* all events send to this element directly. This is mainly done from the
+ * application.
  */
 static gboolean
 gst_base_src_send_event (GstElement * element, GstEvent * event)
@@ -1185,11 +1275,14 @@ gst_base_src_send_event (GstElement * element, GstEvent * event)
   src = GST_BASE_SRC (element);
 
   switch (GST_EVENT_TYPE (event)) {
+      /* bidirectional events */
     case GST_EVENT_FLUSH_START:
     case GST_EVENT_FLUSH_STOP:
       /* sending random flushes downstream can break stuff,
        * especially sync since all segment info will get flushed */
       break;
+
+      /* downstream serialized events */
     case GST_EVENT_EOS:
       /* FIXME, queue EOS and make sure the task or pull function 
        * perform the EOS actions. */
@@ -1198,9 +1291,15 @@ gst_base_src_send_event (GstElement * element, GstEvent * event)
       /* sending random NEWSEGMENT downstream can break sync. */
       break;
     case GST_EVENT_TAG:
-    case GST_EVENT_BUFFERSIZE:
+      /* sending tags could be useful, FIXME insert in dataflow */
       break;
+    case GST_EVENT_BUFFERSIZE:
+      /* does not seem to make much sense currently */
+      break;
+
+      /* upstream events */
     case GST_EVENT_QOS:
+      /* elements should override send_event and do something */
       break;
     case GST_EVENT_SEEK:
     {
@@ -1231,12 +1330,36 @@ gst_base_src_send_event (GstElement * element, GstEvent * event)
       break;
     }
     case GST_EVENT_NAVIGATION:
+      /* could make sense for elements that do something with navigation events
+       * but then they would need to override the send_event function */
+      break;
+    case GST_EVENT_LATENCY:
+      /* does not seem to make sense currently */
+      break;
+
+      /* custom events */
+    case GST_EVENT_CUSTOM_UPSTREAM:
+      /* override send_event if you want this */
+      break;
+    case GST_EVENT_CUSTOM_DOWNSTREAM:
+    case GST_EVENT_CUSTOM_BOTH:
+      /* FIXME, insert event in the dataflow */
+      break;
+    case GST_EVENT_CUSTOM_DOWNSTREAM_OOB:
+    case GST_EVENT_CUSTOM_BOTH_OOB:
+      /* insert a random custom event into the pipeline */
+      GST_DEBUG_OBJECT (src, "pushing custom OOB event downstream");
+      result = gst_pad_push_event (src->srcpad, event);
+      /* we gave away the ref to the event in the push */
+      event = NULL;
       break;
     default:
       break;
   }
 done:
-  gst_event_unref (event);
+  /* if we still have a ref to the event, unref it now */
+  if (event)
+    gst_event_unref (event);
 
   return result;
 
@@ -1266,10 +1389,10 @@ gst_base_src_default_event (GstBaseSrc * src, GstEvent * event)
     case GST_EVENT_FLUSH_START:
       /* cancel any blocking getrange, is normally called
        * when in pull mode. */
-      result = gst_base_src_unlock (src);
+      result = gst_base_src_set_flushing (src, TRUE, FALSE, TRUE, NULL);
       break;
     case GST_EVENT_FLUSH_STOP:
-      result = gst_base_src_unlock_stop (src);
+      result = gst_base_src_set_flushing (src, FALSE, TRUE, TRUE, NULL);
       break;
     default:
       result = TRUE;
@@ -1332,6 +1455,9 @@ gst_base_src_set_property (GObject * object, guint prop_id,
     case PROP_TYPEFIND:
       src->data.ABI.typefind = g_value_get_boolean (value);
       break;
+    case PROP_DO_TIMESTAMP:
+      src->priv->do_timestamp = g_value_get_boolean (value);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -1356,6 +1482,9 @@ gst_base_src_get_property (GObject * object, guint prop_id, GValue * value,
     case PROP_TYPEFIND:
       g_value_set_boolean (value, src->data.ABI.typefind);
       break;
+    case PROP_DO_TIMESTAMP:
+      g_value_set_boolean (value, src->priv->do_timestamp);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -1364,25 +1493,20 @@ gst_base_src_get_property (GObject * object, guint prop_id, GValue * value,
 
 /* with STREAM_LOCK and LOCK */
 static GstClockReturn
-gst_base_src_wait (GstBaseSrc * basesrc, GstClockTime time)
+gst_base_src_wait (GstBaseSrc * basesrc, GstClock * clock, GstClockTime time)
 {
   GstClockReturn ret;
   GstClockID id;
-  GstClock *clock;
-
-  /* get clock, if no clock, we don't sync */
-  if ((clock = GST_ELEMENT_CLOCK (basesrc)) == NULL)
-    return GST_CLOCK_OK;
 
   id = gst_clock_new_single_shot_id (clock, time);
 
   basesrc->clock_id = id;
-  /* release the object lock while waiting */
-  GST_OBJECT_UNLOCK (basesrc);
+  /* release the live lock while waiting */
+  GST_LIVE_UNLOCK (basesrc);
 
   ret = gst_clock_id_wait (id, NULL);
 
-  GST_OBJECT_LOCK (basesrc);
+  GST_LIVE_LOCK (basesrc);
   gst_clock_id_unref (id);
   basesrc->clock_id = NULL;
 
@@ -1399,6 +1523,9 @@ gst_base_src_do_sync (GstBaseSrc * basesrc, GstBuffer * buffer)
   GstClockTime start, end;
   GstBaseSrcClass *bclass;
   GstClockTime base_time;
+  GstClock *clock;
+  GstClockTime now = GST_CLOCK_TIME_NONE, timestamp;
+  gboolean do_timestamp, first, pseudo_live;
 
   bclass = GST_BASE_SRC_GET_CLASS (basesrc);
 
@@ -1406,30 +1533,139 @@ gst_base_src_do_sync (GstBaseSrc * basesrc, GstBuffer * buffer)
   if (bclass->get_times)
     bclass->get_times (basesrc, buffer, &start, &end);
 
-  /* if we don't have a timestamp, we don't sync */
-  if (!GST_CLOCK_TIME_IS_VALID (start))
-    goto invalid_start;
+  /* get buffer timestamp */
+  timestamp = GST_BUFFER_TIMESTAMP (buffer);
 
-  /* now do clocking */
+  /* grab the lock to prepare for clocking and calculate the startup 
+   * latency. */
   GST_OBJECT_LOCK (basesrc);
+
+  /* if we are asked to sync against the clock we are a pseudo live element */
+  pseudo_live = (start != -1 && basesrc->is_live);
+  /* check for the first buffer */
+  first = (basesrc->priv->latency == -1);
+
+  if (timestamp != -1 && pseudo_live) {
+    GstClockTime latency;
+
+    /* we have a timestamp and a sync time, latency is the diff */
+    if (timestamp <= start)
+      latency = start - timestamp;
+    else
+      latency = 0;
+
+    if (first) {
+      GST_DEBUG_OBJECT (basesrc, "pseudo_live with latency %" GST_TIME_FORMAT,
+          GST_TIME_ARGS (latency));
+      /* first time we calculate latency, just configure */
+      basesrc->priv->latency = latency;
+    } else {
+      if (basesrc->priv->latency != latency) {
+        /* we have a new latency, FIXME post latency message */
+        basesrc->priv->latency = latency;
+        GST_DEBUG_OBJECT (basesrc, "latency changed to %" GST_TIME_FORMAT,
+            GST_TIME_ARGS (latency));
+      }
+    }
+  } else if (first) {
+    GST_DEBUG_OBJECT (basesrc, "no latency needed, live %d, sync %d",
+        basesrc->is_live, start != -1);
+    basesrc->priv->latency = 0;
+  }
+
+  /* get clock, if no clock, we can't sync or do timestamps */
+  if ((clock = GST_ELEMENT_CLOCK (basesrc)) == NULL)
+    goto no_clock;
+
   base_time = GST_ELEMENT_CAST (basesrc)->base_time;
 
+  do_timestamp = basesrc->priv->do_timestamp;
+
+  /* first buffer, calculate the timestamp offset */
+  if (first) {
+    GstClockTime running_time;
+
+    now = gst_clock_get_time (clock);
+    running_time = now - base_time;
+
+    GST_LOG_OBJECT (basesrc,
+        "startup timestamp: %" GST_TIME_FORMAT ", running_time %"
+        GST_TIME_FORMAT, GST_TIME_ARGS (timestamp),
+        GST_TIME_ARGS (running_time));
+
+    if (pseudo_live && timestamp != -1) {
+      /* live source and we need to sync, add startup latency to all timestamps
+       * to get the real running_time. Live sources should always timestamp
+       * according to the current running time. */
+      basesrc->priv->ts_offset = GST_CLOCK_DIFF (timestamp, running_time);
+
+      GST_LOG_OBJECT (basesrc, "live with sync, ts_offset %" GST_TIME_FORMAT,
+          GST_TIME_ARGS (basesrc->priv->ts_offset));
+    } else {
+      basesrc->priv->ts_offset = 0;
+      GST_LOG_OBJECT (basesrc, "no timestamp offset needed");
+    }
+
+    if (!GST_CLOCK_TIME_IS_VALID (timestamp)) {
+      if (do_timestamp)
+        timestamp = running_time;
+      else
+        timestamp = 0;
+
+      GST_BUFFER_TIMESTAMP (buffer) = timestamp;
+
+      GST_LOG_OBJECT (basesrc, "created timestamp: %" GST_TIME_FORMAT,
+          GST_TIME_ARGS (timestamp));
+    }
+
+    /* add the timestamp offset we need for sync */
+    timestamp += basesrc->priv->ts_offset;
+  } else {
+    /* not the first buffer, the timestamp is the diff between the clock and
+     * base_time */
+    if (do_timestamp && !GST_CLOCK_TIME_IS_VALID (timestamp)) {
+      now = gst_clock_get_time (clock);
+
+      GST_BUFFER_TIMESTAMP (buffer) = now - base_time;
+
+      GST_LOG_OBJECT (basesrc, "created timestamp: %" GST_TIME_FORMAT,
+          GST_TIME_ARGS (now - base_time));
+    }
+  }
+
+  /* if we don't have a buffer timestamp, we don't sync */
+  if (!GST_CLOCK_TIME_IS_VALID (start))
+    goto no_sync;
+
+  if (basesrc->is_live && GST_CLOCK_TIME_IS_VALID (timestamp)) {
+    /* for pseudo live sources, add our ts_offset to the timestamp */
+    GST_BUFFER_TIMESTAMP (buffer) += basesrc->priv->ts_offset;
+    start += basesrc->priv->ts_offset;
+  }
+
   GST_LOG_OBJECT (basesrc,
       "waiting for clock, base time %" GST_TIME_FORMAT
       ", stream_start %" GST_TIME_FORMAT,
       GST_TIME_ARGS (base_time), GST_TIME_ARGS (start));
-
-  result = gst_base_src_wait (basesrc, start + base_time);
   GST_OBJECT_UNLOCK (basesrc);
 
+  result = gst_base_src_wait (basesrc, clock, start + base_time);
+
   GST_LOG_OBJECT (basesrc, "clock entry done: %d", result);
 
   return result;
 
   /* special cases */
-invalid_start:
+no_clock:
   {
-    GST_DEBUG_OBJECT (basesrc, "get_times returned invalid start");
+    GST_DEBUG_OBJECT (basesrc, "we have no clock");
+    GST_OBJECT_UNLOCK (basesrc);
+    return GST_CLOCK_OK;
+  }
+no_sync:
+  {
+    GST_DEBUG_OBJECT (basesrc, "no sync needed");
+    GST_OBJECT_UNLOCK (basesrc);
     return GST_CLOCK_OK;
   }
 }
@@ -1504,6 +1740,7 @@ unexpected_length:
   }
 }
 
+/* must be called with LIVE_LOCK */
 static GstFlowReturn
 gst_base_src_get_range (GstBaseSrc * src, guint64 offset, guint length,
     GstBuffer ** buf)
@@ -1541,7 +1778,7 @@ gst_base_src_get_range (GstBaseSrc * src, guint64 offset, guint length,
 
   ret = bclass->create (src, offset, length, buf);
   if (G_UNLIKELY (ret != GST_FLOW_OK))
-    goto done;
+    goto not_ok;
 
   /* no timestamp set and we are at offset 0, we can timestamp with 0 */
   if (offset == 0 && src->segment.time == 0
@@ -1550,6 +1787,11 @@ gst_base_src_get_range (GstBaseSrc * src, guint64 offset, guint length,
 
   /* now sync before pushing the buffer */
   status = gst_base_src_do_sync (src, *buf);
+
+  /* waiting for the clock could have made us flushing */
+  if (src->priv->flushing)
+    goto flushing;
+
   switch (status) {
     case GST_CLOCK_EARLY:
       /* the buffer is too late. We currently don't drop the buffer. */
@@ -1580,13 +1822,19 @@ gst_base_src_get_range (GstBaseSrc * src, guint64 offset, guint length,
       ret = GST_FLOW_ERROR;
       break;
   }
-done:
   return ret;
 
   /* ERROR */
 stopped:
   {
-    GST_DEBUG_OBJECT (src, "wait_playing returned %d", ret);
+    GST_DEBUG_OBJECT (src, "wait_playing returned %d (%s)", ret,
+        gst_flow_get_name (ret));
+    return ret;
+  }
+not_ok:
+  {
+    GST_DEBUG_OBJECT (src, "create returned %d (%s)", ret,
+        gst_flow_get_name (ret));
     return ret;
   }
 not_started:
@@ -1610,6 +1858,13 @@ reached_num_buffers:
     GST_DEBUG_OBJECT (src, "sent all buffers");
     return GST_FLOW_UNEXPECTED;
   }
+flushing:
+  {
+    GST_DEBUG_OBJECT (src, "we are flushing");
+    gst_buffer_unref (*buf);
+    *buf = NULL;
+    return GST_FLOW_WRONG_STATE;
+  }
 }
 
 static GstFlowReturn
@@ -1621,11 +1876,26 @@ gst_base_src_pad_get_range (GstPad * pad, guint64 offset, guint length,
 
   src = GST_BASE_SRC (gst_pad_get_parent (pad));
 
+  GST_LIVE_LOCK (src);
+  if (src->priv->flushing)
+    goto flushing;
+
   res = gst_base_src_get_range (src, offset, length, buf);
 
+done:
+  GST_LIVE_UNLOCK (src);
+
   gst_object_unref (src);
 
   return res;
+
+  /* ERRORS */
+flushing:
+  {
+    GST_DEBUG_OBJECT (src, "we are flushing");
+    res = GST_FLOW_WRONG_STATE;
+    goto done;
+  }
 }
 
 static gboolean
@@ -1700,6 +1970,10 @@ gst_base_src_loop (GstPad * pad)
 
   src = GST_BASE_SRC (gst_pad_get_parent (pad));
 
+  GST_LIVE_LOCK (src);
+  if (src->priv->flushing)
+    goto flushing;
+
   src->priv->last_sent_eos = FALSE;
 
   /* if we operate in bytes, we can calculate an offset */
@@ -1712,6 +1986,7 @@ gst_base_src_loop (GstPad * pad)
   if (G_UNLIKELY (ret != GST_FLOW_OK)) {
     GST_INFO_OBJECT (src, "pausing after gst_base_src_get_range() = %s",
         gst_flow_get_name (ret));
+    GST_LIVE_UNLOCK (src);
     goto pause;
   }
   /* this should not happen */
@@ -1771,6 +2046,7 @@ gst_base_src_loop (GstPad * pad)
     GST_BUFFER_FLAG_SET (buf, GST_BUFFER_FLAG_DISCONT);
     src->priv->discont = FALSE;
   }
+  GST_LIVE_UNLOCK (src);
 
   ret = gst_pad_push (pad, buf);
   if (G_UNLIKELY (ret != GST_FLOW_OK)) {
@@ -1790,6 +2066,13 @@ done:
   return;
 
   /* special cases */
+flushing:
+  {
+    GST_DEBUG_OBJECT (src, "we are flushing");
+    GST_LIVE_UNLOCK (src);
+    ret = GST_FLOW_WRONG_STATE;
+    goto pause;
+  }
 pause:
   {
     const gchar *reason = gst_flow_get_name (ret);
@@ -1824,65 +2107,13 @@ null_buffer:
   {
     GST_ELEMENT_ERROR (src, STREAM, FAILED,
         (_("Internal data flow error.")), ("element returned NULL buffer"));
+    GST_LIVE_UNLOCK (src);
     /* we finished the segment on error */
-    src->data.ABI.running = FALSE;
-    gst_pad_pause_task (pad);
-    gst_pad_push_event (pad, gst_event_new_eos ());
-    src->priv->last_sent_eos = TRUE;
+    ret = GST_FLOW_ERROR;
     goto done;
   }
 }
 
-/* this will always be called between start() and stop(). So you can rely on
- * resources allocated by start() and freed from stop(). This needs to be added
- * to the docs at some point. */
-static gboolean
-gst_base_src_unlock (GstBaseSrc * basesrc)
-{
-  GstBaseSrcClass *bclass;
-  gboolean result = TRUE;
-
-  GST_DEBUG ("unlock");
-  /* unblock whatever the subclass is doing */
-  bclass = GST_BASE_SRC_GET_CLASS (basesrc);
-  if (bclass->unlock)
-    result = bclass->unlock (basesrc);
-
-  GST_DEBUG ("unschedule clock");
-  /* and unblock the clock as well, if any */
-  GST_OBJECT_LOCK (basesrc);
-  if (basesrc->clock_id) {
-    gst_clock_id_unschedule (basesrc->clock_id);
-  }
-  GST_OBJECT_UNLOCK (basesrc);
-
-  GST_DEBUG ("unlock done");
-
-  return result;
-}
-
-/* this will always be called between start() and stop(). So you can rely on
- * resources allocated by start() and freed from stop(). This needs to be added
- * to the docs at some point. */
-static gboolean
-gst_base_src_unlock_stop (GstBaseSrc * basesrc)
-{
-  GstBaseSrcClass *bclass;
-  gboolean result = TRUE;
-
-  GST_DEBUG_OBJECT (basesrc, "unlock stop");
-
-  /* Finish a previous unblock request, allowing subclasses to flush command
-   * queues or whatever they need to do */
-  bclass = GST_BASE_SRC_GET_CLASS (basesrc);
-  if (bclass->unlock_stop)
-    result = bclass->unlock_stop (basesrc);
-
-  GST_DEBUG_OBJECT (basesrc, "unlock stop done");
-
-  return result;
-}
-
 /* default negotiation code. 
  *
  * Take intersection between src and sink pads, take first
@@ -2087,26 +2318,102 @@ gst_base_src_stop (GstBaseSrc * basesrc)
   return result;
 }
 
+/* start or stop flushing dataprocessing 
+ */
 static gboolean
-gst_base_src_deactivate (GstBaseSrc * basesrc, GstPad * pad)
+gst_base_src_set_flushing (GstBaseSrc * basesrc,
+    gboolean flushing, gboolean live_play, gboolean unlock, gboolean * playing)
 {
-  gboolean result;
+  GstBaseSrcClass *bclass;
 
+  bclass = GST_BASE_SRC_GET_CLASS (basesrc);
+
+  if (flushing && unlock) {
+    /* unlock any subclasses, we need to do this before grabbing the
+     * LIVE_LOCK since we hold this lock before going into ::create. We pass an
+     * unlock to the params because of backwards compat (see seek handler)*/
+    if (bclass->unlock)
+      bclass->unlock (basesrc);
+  }
+
+  /* the live lock is released when we are blocked, waiting for playing or
+   * when we sync to the clock. */
   GST_LIVE_LOCK (basesrc);
-  basesrc->live_running = TRUE;
+  if (playing)
+    *playing = basesrc->live_running;
+  basesrc->priv->flushing = flushing;
+  if (flushing) {
+    /* if we are locked in the live lock, signal it to make it flush */
+    basesrc->live_running = TRUE;
+
+    /* step 1, now that we have the LIVE lock, clear our unlock request */
+    if (bclass->unlock_stop)
+      bclass->unlock_stop (basesrc);
+
+    /* step 2, unblock clock sync (if any) or any other blocking thing */
+    if (basesrc->clock_id)
+      gst_clock_id_unschedule (basesrc->clock_id);
+  } else {
+    /* signal the live source that it can start playing */
+    basesrc->live_running = live_play;
+  }
   GST_LIVE_SIGNAL (basesrc);
   GST_LIVE_UNLOCK (basesrc);
 
-  /* step 1, unblock clock sync (if any) */
-  result = gst_base_src_unlock (basesrc);
+  return TRUE;
+}
 
-  /* step 2, make sure streaming finishes */
-  result &= gst_pad_stop_task (pad);
+/* the purpose of this function is to make sure that a live source blocks in the
+ * LIVE lock or leaves the LIVE lock and continues playing. */
+static gboolean
+gst_base_src_set_playing (GstBaseSrc * basesrc, gboolean live_play)
+{
+  GstBaseSrcClass *bclass;
 
-  /* step 3, clear the unblock condition */
-  result &= gst_base_src_unlock_stop (basesrc);
+  bclass = GST_BASE_SRC_GET_CLASS (basesrc);
 
-  return result;
+  /* unlock subclasses locked in ::create, we only do this when we stop playing. */
+  if (!live_play) {
+    GST_DEBUG_OBJECT (basesrc, "unlock");
+    if (bclass->unlock)
+      bclass->unlock (basesrc);
+  }
+
+  /* we are now able to grab the LIVE lock, when we get it, we can be
+   * waiting for PLAYING while blocked in the LIVE cond or we can be waiting
+   * for the clock. */
+  GST_LIVE_LOCK (basesrc);
+
+  /* unblock clock sync (if any) */
+  if (basesrc->clock_id)
+    gst_clock_id_unschedule (basesrc->clock_id);
+
+  /* configure what to do when we get to the LIVE lock. */
+  basesrc->live_running = live_play;
+
+  if (live_play) {
+    gboolean start;
+
+    /* clear our unlock request when going to PLAYING */
+    GST_DEBUG_OBJECT (basesrc, "unlock stop");
+    if (bclass->unlock_stop)
+      bclass->unlock_stop (basesrc);
+
+    /* for live sources we restart the timestamp correction */
+    basesrc->priv->latency = -1;
+    /* have to restart the task in case it stopped because of the unlock when
+     * we went to PAUSED. Only do this if we operating in push mode. */
+    GST_OBJECT_LOCK (basesrc->srcpad);
+    start = (GST_PAD_ACTIVATE_MODE (basesrc->srcpad) == GST_ACTIVATE_PUSH);
+    GST_OBJECT_UNLOCK (basesrc->srcpad);
+    if (start)
+      gst_pad_start_task (basesrc->srcpad, (GstTaskFunction) gst_base_src_loop,
+          basesrc->srcpad);
+  }
+  GST_LIVE_SIGNAL (basesrc);
+  GST_LIVE_UNLOCK (basesrc);
+
+  return TRUE;
 }
 
 static gboolean
@@ -2128,6 +2435,8 @@ gst_base_src_activate_push (GstPad * pad, gboolean active)
       goto error_start;
 
     basesrc->priv->last_sent_eos = FALSE;
+    basesrc->priv->discont = TRUE;
+    gst_base_src_set_flushing (basesrc, FALSE, FALSE, FALSE, NULL);
 
     /* do initial seek, which will start the task */
     GST_OBJECT_LOCK (basesrc);
@@ -2145,10 +2454,10 @@ gst_base_src_activate_push (GstPad * pad, gboolean active)
       gst_event_unref (event);
   } else {
     GST_DEBUG_OBJECT (basesrc, "Deactivating in push mode");
-    /* call the unlock function and stop the task */
-    if (G_UNLIKELY (!gst_base_src_deactivate (basesrc, pad)))
-      goto deactivate_failed;
-
+    /* flush all */
+    gst_base_src_set_flushing (basesrc, TRUE, FALSE, TRUE, NULL);
+    /* stop the task */
+    gst_pad_stop_task (pad);
     /* now we can stop the source */
     if (G_UNLIKELY (!gst_base_src_stop (basesrc)))
       goto error_stop;
@@ -2174,11 +2483,6 @@ seek_failed:
       gst_event_unref (event);
     return FALSE;
   }
-deactivate_failed:
-  {
-    GST_ERROR_OBJECT (basesrc, "Failed to deactivate in push mode");
-    return FALSE;
-  }
 error_stop:
   {
     GST_DEBUG_OBJECT (basesrc, "Failed to stop in push mode");
@@ -2202,11 +2506,14 @@ gst_base_src_activate_pull (GstPad * pad, gboolean active)
     /* if not random_access, we cannot operate in pull mode for now */
     if (G_UNLIKELY (!gst_base_src_check_get_range (basesrc)))
       goto no_get_range;
+
+    /* stop flushing now but for live sources, still block in the LIVE lock when
+     * we are not yet PLAYING */
+    gst_base_src_set_flushing (basesrc, FALSE, FALSE, FALSE, NULL);
   } else {
     GST_DEBUG_OBJECT (basesrc, "Deactivating in pull mode");
-    /* call the unlock function. We have no task to stop. */
-    if (G_UNLIKELY (!gst_base_src_deactivate (basesrc, pad)))
-      goto deactivate_failed;
+    /* flush all, there is no task to stop */
+    gst_base_src_set_flushing (basesrc, TRUE, FALSE, TRUE, NULL);
 
     /* don't send EOS when going from PAUSED => READY when in pull mode */
     basesrc->priv->last_sent_eos = TRUE;
@@ -2228,11 +2535,6 @@ no_get_range:
     gst_base_src_stop (basesrc);
     return FALSE;
   }
-deactivate_failed:
-  {
-    GST_ERROR_OBJECT (basesrc, "Failed to deactivate in pull mode");
-    return FALSE;
-  }
 error_stop:
   {
     GST_ERROR_OBJECT (basesrc, "Failed to stop in pull mode");
@@ -2253,22 +2555,13 @@ gst_base_src_change_state (GstElement * element, GstStateChange transition)
     case GST_STATE_CHANGE_NULL_TO_READY:
       break;
     case GST_STATE_CHANGE_READY_TO_PAUSED:
-      GST_LIVE_LOCK (element);
-      if (basesrc->is_live) {
-        no_preroll = TRUE;
-        basesrc->live_running = FALSE;
-      }
-      basesrc->priv->last_sent_eos = FALSE;
-      basesrc->priv->discont = TRUE;
-      GST_LIVE_UNLOCK (element);
+      no_preroll = gst_base_src_is_live (basesrc);
       break;
     case GST_STATE_CHANGE_PAUSED_TO_PLAYING:
-      GST_LIVE_LOCK (element);
-      if (basesrc->is_live) {
-        basesrc->live_running = TRUE;
-        GST_LIVE_SIGNAL (element);
+      if (gst_base_src_is_live (basesrc)) {
+        /* now we can start playback */
+        gst_base_src_set_playing (basesrc, TRUE);
       }
-      GST_LIVE_UNLOCK (element);
       break;
     default:
       break;
@@ -2281,17 +2574,19 @@ gst_base_src_change_state (GstElement * element, GstStateChange transition)
 
   switch (transition) {
     case GST_STATE_CHANGE_PLAYING_TO_PAUSED:
-      GST_LIVE_LOCK (element);
-      if (basesrc->is_live) {
+      if (gst_base_src_is_live (basesrc)) {
+        /* make sure we block in the live lock in PAUSED */
+        gst_base_src_set_playing (basesrc, FALSE);
         no_preroll = TRUE;
-        basesrc->live_running = FALSE;
       }
-      GST_LIVE_UNLOCK (element);
       break;
     case GST_STATE_CHANGE_PAUSED_TO_READY:
     {
       GstEvent **event_p;
 
+      /* we don't need to unblock anything here, the pad deactivation code
+       * already did this */
+
       /* FIXME, deprecate this behaviour, it is very dangerous.
        * the prefered way of sending EOS downstream is by sending
        * the EOS event to the element */
