@@ -118,8 +118,8 @@ enum
   ARG_MIN_THRESHOLD_BUFFERS,
   ARG_MIN_THRESHOLD_BYTES,
   ARG_MIN_THRESHOLD_TIME,
-  ARG_LEAKY,
-  /* FILL ME */
+  ARG_LEAKY
+      /* FILL ME */
 };
 
 /* default property values */
@@ -142,23 +142,23 @@ enum
 } G_STMT_END
 
 #define GST_QUEUE_WAIT_DEL_CHECK(q, label) G_STMT_START {               \
-  STATUS (queue, q->sinkpad, "wait for DEL");                           \
-  g_cond_wait (q->item_del, queue->qlock);                              \
+  STATUS (q, q->sinkpad, "wait for DEL");                               \
+  g_cond_wait (q->item_del, q->qlock);                                  \
   if (q->srcresult != GST_FLOW_OK) {                                    \
-    STATUS (queue, q->srcpad, "received DEL wakeup");                   \
+    STATUS (q, q->srcpad, "received DEL wakeup");                       \
     goto label;                                                         \
   }                                                                     \
-  STATUS (queue, q->sinkpad, "received DEL");                           \
+  STATUS (q, q->sinkpad, "received DEL");                               \
 } G_STMT_END
 
 #define GST_QUEUE_WAIT_ADD_CHECK(q, label) G_STMT_START {               \
-  STATUS (queue, q->srcpad, "wait for ADD");                            \
+  STATUS (q, q->srcpad, "wait for ADD");                                \
   g_cond_wait (q->item_add, q->qlock);                                  \
   if (q->srcresult != GST_FLOW_OK) {                                    \
-    STATUS (queue, q->srcpad, "received ADD wakeup");                   \
+    STATUS (q, q->srcpad, "received ADD wakeup");                       \
     goto label;                                                         \
   }                                                                     \
-  STATUS (queue, q->srcpad, "received ADD");                            \
+  STATUS (q, q->srcpad, "received ADD");                                \
 } G_STMT_END
 
 #define GST_QUEUE_SIGNAL_DEL(q) G_STMT_START {                          \
@@ -189,7 +189,6 @@ static void gst_queue_get_property (GObject * object,
 static GstFlowReturn gst_queue_chain (GstPad * pad, GstBuffer * buffer);
 static GstFlowReturn gst_queue_bufferalloc (GstPad * pad, guint64 offset,
     guint size, GstCaps * caps, GstBuffer ** buf);
-static gboolean gst_queue_acceptcaps (GstPad * pad, GstCaps * caps);
 static GstFlowReturn gst_queue_push_one (GstQueue * queue);
 static void gst_queue_loop (GstPad * pad);
 
@@ -383,8 +382,6 @@ gst_queue_init (GstQueue * queue, GstQueueClass * g_class)
       GST_DEBUG_FUNCPTR (gst_queue_src_activate_push));
   gst_pad_set_link_function (queue->srcpad,
       GST_DEBUG_FUNCPTR (gst_queue_link_src));
-  gst_pad_set_acceptcaps_function (queue->srcpad,
-      GST_DEBUG_FUNCPTR (gst_queue_acceptcaps));
   gst_pad_set_getcaps_function (queue->srcpad,
       GST_DEBUG_FUNCPTR (gst_queue_getcaps));
   gst_pad_set_event_function (queue->srcpad,
@@ -503,14 +500,6 @@ gst_queue_bufferalloc (GstPad * pad, guint64 offset, guint size, GstCaps * caps,
   return result;
 }
 
-static gboolean
-gst_queue_acceptcaps (GstPad * pad, GstCaps * caps)
-{
-  /* The only time our acceptcaps method should be called is on the srcpad
-   * when we push a buffer, in which case we always accept those caps */
-  return TRUE;
-}
-
 /* calculate the diff between running time on the sink and src of the queue.
  * This is the total amount of time in the queue. */
 static void
@@ -525,7 +514,7 @@ update_time_level (GstQueue * queue)
   src_time = gst_segment_to_running_time (&queue->src_segment, GST_FORMAT_TIME,
       queue->src_segment.last_stop);
 
-  GST_DEBUG_OBJECT (queue, "sink %" GST_TIME_FORMAT ", src %" GST_TIME_FORMAT,
+  GST_LOG_OBJECT (queue, "sink %" GST_TIME_FORMAT ", src %" GST_TIME_FORMAT,
       GST_TIME_ARGS (sink_time), GST_TIME_ARGS (src_time));
 
   if (sink_time >= src_time)
@@ -570,7 +559,8 @@ apply_segment (GstQueue * queue, GstEvent * event, GstSegment * segment)
 
 /* take a buffer and update segment, updating the time level of the queue. */
 static void
-apply_buffer (GstQueue * queue, GstBuffer * buffer, GstSegment * segment)
+apply_buffer (GstQueue * queue, GstBuffer * buffer, GstSegment * segment,
+    gboolean with_duration)
 {
   GstClockTime duration, timestamp;
 
@@ -583,10 +573,10 @@ apply_buffer (GstQueue * queue, GstBuffer * buffer, GstSegment * segment)
     timestamp = segment->last_stop;
 
   /* add duration */
-  if (duration != GST_CLOCK_TIME_NONE)
+  if (with_duration && duration != GST_CLOCK_TIME_NONE)
     timestamp += duration;
 
-  GST_DEBUG_OBJECT (queue, "last_stop updated to %" GST_TIME_FORMAT,
+  GST_LOG_OBJECT (queue, "last_stop updated to %" GST_TIME_FORMAT,
       GST_TIME_ARGS (timestamp));
 
   gst_segment_set_last_stop (segment, GST_FORMAT_TIME, timestamp);
@@ -616,7 +606,7 @@ gst_queue_locked_flush (GstQueue * queue)
   GST_QUEUE_SIGNAL_DEL (queue);
 }
 
-/* enqueue an item an update the level stats */
+/* enqueue an item an update the level stats, with QUEUE_LOCK */
 static void
 gst_queue_locked_enqueue (GstQueue * queue, gpointer item)
 {
@@ -626,8 +616,16 @@ gst_queue_locked_enqueue (GstQueue * queue, gpointer item)
     /* add buffer to the statistics */
     queue->cur_level.buffers++;
     queue->cur_level.bytes += GST_BUFFER_SIZE (buffer);
-    apply_buffer (queue, buffer, &queue->sink_segment);
+    apply_buffer (queue, buffer, &queue->sink_segment, TRUE);
 
+    /* if this is the first buffer update the end side as well, but without the
+     * duration. */
+    /* FIXME : This will only be useful for current time level if the
+     * source task is running, which is not the case for ex in 
+     * gstplaybasebin when pre-rolling.
+     * See #482147 */
+    /*     if (queue->cur_level.buffers == 1) */
+    /*       apply_buffer (queue, buffer, &queue->src_segment, FALSE); */
   } else if (GST_IS_EVENT (item)) {
     GstEvent *event = GST_EVENT_CAST (item);
 
@@ -636,9 +634,15 @@ gst_queue_locked_enqueue (GstQueue * queue, gpointer item)
         /* Zero the thresholds, this makes sure the queue is completely
          * filled and we can read all data from the queue. */
         GST_QUEUE_CLEAR_LEVEL (queue->min_threshold);
+        /* mark the queue as EOS. This prevents us from accepting more data. */
+        GST_CAT_LOG_OBJECT (queue_dataflow, queue, "got EOS from upstream");
+        queue->eos = TRUE;
         break;
       case GST_EVENT_NEWSEGMENT:
         apply_segment (queue, event, &queue->sink_segment);
+        /* a new segment allows us to accept more buffers if we got UNEXPECTED
+         * from downstream */
+        queue->unexpected = FALSE;
         break;
       default:
         break;
@@ -655,7 +659,7 @@ gst_queue_locked_enqueue (GstQueue * queue, gpointer item)
   GST_QUEUE_SIGNAL_ADD (queue);
 }
 
-/* dequeue an item from the queue and update level stats */
+/* dequeue an item from the queue and update level stats, with QUEUE_LOCK */
 static GstMiniObject *
 gst_queue_locked_dequeue (GstQueue * queue)
 {
@@ -673,8 +677,11 @@ gst_queue_locked_dequeue (GstQueue * queue)
 
     queue->cur_level.buffers--;
     queue->cur_level.bytes -= GST_BUFFER_SIZE (buffer);
-    apply_buffer (queue, buffer, &queue->src_segment);
+    apply_buffer (queue, buffer, &queue->src_segment, TRUE);
 
+    /* if the queue is empty now, update the other side */
+    if (queue->cur_level.buffers == 0)
+      queue->cur_level.time = 0;
   } else if (GST_IS_EVENT (item)) {
     GstEvent *event = GST_EVENT_CAST (item);
 
@@ -705,7 +712,7 @@ gst_queue_locked_dequeue (GstQueue * queue)
   /* ERRORS */
 no_item:
   {
-    GST_CAT_LOG_OBJECT (queue_dataflow, queue, "the queue is empty");
+    GST_CAT_DEBUG_OBJECT (queue_dataflow, queue, "the queue is empty");
     return NULL;
   }
 }
@@ -728,8 +735,8 @@ gst_queue_handle_sink_event (GstPad * pad, GstEvent * event)
       GST_QUEUE_MUTEX_LOCK (queue);
       queue->srcresult = GST_FLOW_WRONG_STATE;
       /* unblock the loop and chain functions */
-      g_cond_signal (queue->item_add);
-      g_cond_signal (queue->item_del);
+      GST_QUEUE_SIGNAL_ADD (queue);
+      GST_QUEUE_SIGNAL_DEL (queue);
       GST_QUEUE_MUTEX_UNLOCK (queue);
 
       /* make sure it pauses, this should happen since we sent
@@ -747,11 +754,13 @@ gst_queue_handle_sink_event (GstPad * pad, GstEvent * event)
       GST_QUEUE_MUTEX_LOCK (queue);
       gst_queue_locked_flush (queue);
       queue->srcresult = GST_FLOW_OK;
+      queue->eos = FALSE;
+      queue->unexpected = FALSE;
       if (gst_pad_is_linked (queue->srcpad)) {
         gst_pad_start_task (queue->srcpad, (GstTaskFunction) gst_queue_loop,
             queue->srcpad);
       } else {
-        GST_DEBUG_OBJECT (queue, "not re-starting task as pad is not linked");
+        GST_INFO_OBJECT (queue, "not re-starting task as pad is not linked");
       }
       GST_QUEUE_MUTEX_UNLOCK (queue);
 
@@ -762,6 +771,9 @@ gst_queue_handle_sink_event (GstPad * pad, GstEvent * event)
       if (GST_EVENT_IS_SERIALIZED (event)) {
         /* serialized events go in the queue */
         GST_QUEUE_MUTEX_LOCK_CHECK (queue, out_flushing);
+        /* refuse more events on EOS */
+        if (queue->eos)
+          goto out_eos;
         gst_queue_locked_enqueue (queue, event);
         GST_QUEUE_MUTEX_UNLOCK (queue);
       } else {
@@ -776,6 +788,15 @@ done:
   /* ERRORS */
 out_flushing:
   {
+    GST_CAT_LOG_OBJECT (queue_dataflow, queue,
+        "refusing event, we are flushing");
+    GST_QUEUE_MUTEX_UNLOCK (queue);
+    gst_buffer_unref (event);
+    return FALSE;
+  }
+out_eos:
+  {
+    GST_CAT_LOG_OBJECT (queue_dataflow, queue, "refusing event, we are EOS");
     GST_QUEUE_MUTEX_UNLOCK (queue);
     gst_buffer_unref (event);
     return FALSE;
@@ -815,6 +836,11 @@ gst_queue_chain (GstPad * pad, GstBuffer * buffer)
 
   /* we have to lock the queue since we span threads */
   GST_QUEUE_MUTEX_LOCK_CHECK (queue, out_flushing);
+  /* when we received EOS, we refuse any more data */
+  if (queue->eos)
+    goto out_eos;
+  if (queue->unexpected)
+    goto out_unexpected;
 
   timestamp = GST_BUFFER_TIMESTAMP (buffer);
   duration = GST_BUFFER_DURATION (buffer);
@@ -876,6 +902,8 @@ gst_queue_chain (GstPad * pad, GstBuffer * buffer)
           GST_QUEUE_WAIT_DEL_CHECK (queue, out_flushing);
         } while (gst_queue_is_filled (queue));
 
+        GST_CAT_DEBUG_OBJECT (queue_dataflow, queue, "queue is not full");
+
         GST_QUEUE_MUTEX_UNLOCK (queue);
         g_signal_emit (G_OBJECT (queue), gst_queue_signals[SIGNAL_RUNNING], 0);
         GST_QUEUE_MUTEX_LOCK_CHECK (queue, out_flushing);
@@ -910,6 +938,25 @@ out_flushing:
 
     return ret;
   }
+out_eos:
+  {
+    GST_CAT_LOG_OBJECT (queue_dataflow, queue, "exit because we received EOS");
+    GST_QUEUE_MUTEX_UNLOCK (queue);
+
+    gst_buffer_unref (buffer);
+
+    return GST_FLOW_UNEXPECTED;
+  }
+out_unexpected:
+  {
+    GST_CAT_LOG_OBJECT (queue_dataflow, queue,
+        "exit because we received UNEXPECTED");
+    GST_QUEUE_MUTEX_UNLOCK (queue);
+
+    gst_buffer_unref (buffer);
+
+    return GST_FLOW_UNEXPECTED;
+  }
 }
 
 /* dequeue an item from the queue an push it downstream. This functions returns
@@ -924,15 +971,63 @@ gst_queue_push_one (GstQueue * queue)
   if (data == NULL)
     goto no_item;
 
+next:
   if (GST_IS_BUFFER (data)) {
-    GstBuffer *buffer = GST_BUFFER_CAST (data);
+    GstBuffer *buffer;
+    GstCaps *caps;
+
+    buffer = GST_BUFFER_CAST (data);
+    caps = GST_BUFFER_CAPS (buffer);
 
     GST_QUEUE_MUTEX_UNLOCK (queue);
+    /* set the right caps on the pad now. We do this before pushing the buffer
+     * because the pad_push call will check (using acceptcaps) if the buffer can
+     * be set on the pad, which might fail because this will be propagated
+     * upstream. Also note that if the buffer has NULL caps, it means that the
+     * caps did not change, so we don't have to change caps on the pad. */
+    if (caps && caps != GST_PAD_CAPS (queue->srcpad))
+      gst_pad_set_caps (queue->srcpad, caps);
 
     result = gst_pad_push (queue->srcpad, buffer);
 
     /* need to check for srcresult here as well */
     GST_QUEUE_MUTEX_LOCK_CHECK (queue, out_flushing);
+
+    if (result == GST_FLOW_UNEXPECTED) {
+      GST_CAT_LOG_OBJECT (queue_dataflow, queue,
+          "got UNEXPECTED from downstream");
+      /* stop pushing buffers, we dequeue all items until we see an item that we
+       * can push again, which is EOS or NEWSEGMENT. If there is nothing in the
+       * queue we can push, we set a flag to make the sinkpad refuse more
+       * buffers with an UNEXPECTED return value. */
+      while ((data = gst_queue_locked_dequeue (queue))) {
+        if (GST_IS_BUFFER (data)) {
+          GST_CAT_LOG_OBJECT (queue_dataflow, queue,
+              "dropping UNEXPECTED buffer %p", data);
+          gst_buffer_unref (GST_BUFFER_CAST (data));
+        } else if (GST_IS_EVENT (data)) {
+          GstEvent *event = GST_EVENT_CAST (data);
+          GstEventType type = GST_EVENT_TYPE (event);
+
+          if (type == GST_EVENT_EOS || type == GST_EVENT_NEWSEGMENT) {
+            /* we found a pushable item in the queue, push it out */
+            GST_CAT_LOG_OBJECT (queue_dataflow, queue,
+                "pushing pushable event %s after UNEXPECTED",
+                GST_EVENT_TYPE_NAME (event));
+            goto next;
+          }
+          GST_CAT_LOG_OBJECT (queue_dataflow, queue,
+              "dropping UNEXPECTED event %p", event);
+          gst_event_unref (event);
+        }
+      }
+      /* no more items in the queue. Set the unexpected flag so that upstream
+       * make us refuse any more buffers on the sinkpad. Since we will still
+       * accept EOS and NEWSEGMENT we return _FLOW_OK to the caller so that the
+       * task function does not shut down. */
+      queue->unexpected = TRUE;
+      result = GST_FLOW_OK;
+    }
   } else if (GST_IS_EVENT (data)) {
     GstEvent *event = GST_EVENT_CAST (data);
     GstEventType type = GST_EVENT_TYPE (event);
@@ -943,8 +1038,11 @@ gst_queue_push_one (GstQueue * queue)
 
     GST_QUEUE_MUTEX_LOCK_CHECK (queue, out_flushing);
     /* if we're EOS, return UNEXPECTED so that the task pauses. */
-    if (type == GST_EVENT_EOS)
+    if (type == GST_EVENT_EOS) {
+      GST_CAT_LOG_OBJECT (queue_dataflow, queue,
+          "pushed EOS event %p, return UNEXPECTED", event);
       result = GST_FLOW_UNEXPECTED;
+    }
   }
   return result;
 
@@ -976,6 +1074,7 @@ gst_queue_loop (GstPad * pad)
   while (gst_queue_is_empty (queue)) {
     GST_QUEUE_MUTEX_UNLOCK (queue);
     g_signal_emit (G_OBJECT (queue), gst_queue_signals[SIGNAL_UNDERRUN], 0);
+    GST_CAT_DEBUG_OBJECT (queue_dataflow, queue, "queue is empty");
     GST_QUEUE_MUTEX_LOCK_CHECK (queue, out_flushing);
 
     /* we recheck, the signal could have changed the thresholds */
@@ -986,6 +1085,7 @@ gst_queue_loop (GstPad * pad)
 
     g_signal_emit (G_OBJECT (queue), gst_queue_signals[SIGNAL_RUNNING], 0);
     g_signal_emit (G_OBJECT (queue), gst_queue_signals[SIGNAL_PUSHING], 0);
+    GST_CAT_DEBUG_OBJECT (queue_dataflow, queue, "queue is not empty");
 
     GST_QUEUE_MUTEX_LOCK_CHECK (queue, out_flushing);
   }
@@ -1068,6 +1168,25 @@ gst_queue_handle_src_query (GstPad * pad, GstQuery * query)
       gst_query_set_position (query, format, peer_pos);
       break;
     }
+    case GST_QUERY_LATENCY:
+    {
+      gboolean live;
+      GstClockTime min, max;
+
+      gst_query_parse_latency (query, &live, &min, &max);
+
+      /* we can delay up to the limit of the queue in time. If we have no time
+       * limit, the best thing we can do is to return an infinite delay. In
+       * reality a better estimate would be the byte/buffer rate but that is not
+       * possible right now. */
+      if (queue->max_size.time > 0 && max != -1)
+        max += queue->max_size.time;
+      else
+        max = -1;
+
+      gst_query_set_latency (query, live, min, max);
+      break;
+    }
     default:
       /* peer handled other queries */
       break;
@@ -1087,6 +1206,8 @@ gst_queue_sink_activate_push (GstPad * pad, gboolean active)
   if (active) {
     GST_QUEUE_MUTEX_LOCK (queue);
     queue->srcresult = GST_FLOW_OK;
+    queue->eos = FALSE;
+    queue->unexpected = FALSE;
     GST_QUEUE_MUTEX_UNLOCK (queue);
   } else {
     /* step 1, unblock chain function */
@@ -1112,11 +1233,13 @@ gst_queue_src_activate_push (GstPad * pad, gboolean active)
   if (active) {
     GST_QUEUE_MUTEX_LOCK (queue);
     queue->srcresult = GST_FLOW_OK;
+    queue->eos = FALSE;
+    queue->unexpected = FALSE;
     /* we do not start the task yet if the pad is not connected */
     if (gst_pad_is_linked (pad))
       result = gst_pad_start_task (pad, (GstTaskFunction) gst_queue_loop, pad);
     else {
-      GST_DEBUG_OBJECT (queue, "not starting task as pad is not linked");
+      GST_INFO_OBJECT (queue, "not starting task as pad is not linked");
       result = TRUE;
     }
     GST_QUEUE_MUTEX_UNLOCK (queue);
@@ -1176,14 +1299,14 @@ gst_queue_change_state (GstElement * element, GstStateChange transition)
  * the _chain function, it might have more room now
  * to store the buffer/event in the queue */
 #define QUEUE_CAPACITY_CHANGE(q)\
-  g_cond_signal (queue->item_del);
+  GST_QUEUE_SIGNAL_DEL (q);
 
 /* Changing the minimum required fill level must
  * wake up the _loop function as it might now
  * be able to preceed.
  */
 #define QUEUE_THRESHOLD_CHANGE(q)\
-  g_cond_signal (queue->item_add);
+  GST_QUEUE_SIGNAL_ADD (q);
 
 static void
 gst_queue_set_property (GObject * object,
