@@ -22,6 +22,21 @@
 
 #include <gst/check/gstcheck.h>
 
+static void
+pop_async_done (GstBus * bus)
+{
+  GstMessage *message;
+
+  GST_DEBUG ("popping async-done message");
+  message = gst_bus_poll (bus, GST_MESSAGE_ASYNC_DONE, -1);
+
+  fail_unless (message && GST_MESSAGE_TYPE (message)
+      == GST_MESSAGE_ASYNC_DONE, "did not get GST_MESSAGE_ASYNC_DONE");
+
+  gst_message_unref (message);
+  GST_DEBUG ("popped message");
+}
+
 static void
 pop_messages (GstBus * bus, int count)
 {
@@ -277,7 +292,7 @@ GST_START_TEST (test_message_state_changed_children)
   fail_unless (pending == GST_STATE_VOID_PENDING);
 
   /* wait for async thread to settle down */
-  while (GST_OBJECT_REFCOUNT_VALUE (pipeline) > 2)
+  while (GST_OBJECT_REFCOUNT_VALUE (pipeline) > 3)
     THREAD_SWITCH ();
 
   /* each object is referenced by a message;
@@ -285,12 +300,15 @@ GST_START_TEST (test_message_state_changed_children)
    * base_sink_chain has taken a refcount on the sink, and is blocked on
    * preroll */
   ASSERT_OBJECT_REFCOUNT (src, "src", 3);
-  ASSERT_OBJECT_REFCOUNT (sink, "sink", 3);
+  /* refcount can be 4 if the bin is still processing the async_done message of
+   * the sink. */
+  ASSERT_OBJECT_REFCOUNT_BETWEEN (sink, "sink", 3, 4);
   /* 2 or 3 is valid, because the pipeline might still be posting 
    * its state_change message */
   ASSERT_OBJECT_REFCOUNT_BETWEEN (pipeline, "pipeline", 2, 3);
 
   pop_messages (bus, 3);
+  pop_async_done (bus);
   fail_if ((gst_bus_pop (bus)) != NULL);
 
   ASSERT_OBJECT_REFCOUNT (bus, "bus", 2);
@@ -390,6 +408,7 @@ GST_START_TEST (test_watch_for_state_change)
   fail_unless (ret == GST_STATE_CHANGE_SUCCESS);
 
   pop_messages (bus, 6);
+  pop_async_done (bus);
 
   fail_unless (gst_bus_have_pending (bus) == FALSE,
       "Unexpected messages on bus");
@@ -400,10 +419,12 @@ GST_START_TEST (test_watch_for_state_change)
   pop_messages (bus, 3);
 
   /* this one might return either SUCCESS or ASYNC, likely SUCCESS */
-  gst_element_set_state (GST_ELEMENT (bin), GST_STATE_PAUSED);
+  ret = gst_element_set_state (GST_ELEMENT (bin), GST_STATE_PAUSED);
   gst_element_get_state (GST_ELEMENT (bin), NULL, NULL, GST_CLOCK_TIME_NONE);
 
   pop_messages (bus, 3);
+  if (ret == GST_STATE_CHANGE_ASYNC)
+    pop_async_done (bus);
 
   fail_unless (gst_bus_have_pending (bus) == FALSE,
       "Unexpected messages on bus");
@@ -521,6 +542,7 @@ GST_START_TEST (test_children_state_change_order_flagged_sink)
 
   src = gst_element_factory_make ("fakesrc", NULL);
   fail_if (src == NULL, "Could not create fakesrc");
+  g_object_set (src, "num-buffers", 5, NULL);
 
   identity = gst_element_factory_make ("identity", NULL);
   fail_if (identity == NULL, "Could not create identity");
@@ -552,6 +574,7 @@ GST_START_TEST (test_children_state_change_order_flagged_sink)
   /* READY => PAUSED */
   /* because of pre-rolling, sink will return ASYNC on state
    * change and change state later when it has a buffer */
+  GST_DEBUG ("popping READY -> PAUSED messages");
   ASSERT_STATE_CHANGE_MSG (bus, identity, GST_STATE_READY, GST_STATE_PAUSED,
       105);
 #if 0
@@ -561,20 +584,21 @@ GST_START_TEST (test_children_state_change_order_flagged_sink)
    * in which case the sink will commit its new state before the source ...  */
   ASSERT_STATE_CHANGE_MSG (bus, src, GST_STATE_READY, GST_STATE_PAUSED, 106);
   ASSERT_STATE_CHANGE_MSG (bus, sink, GST_STATE_READY, GST_STATE_PAUSED, 107);
+#else
+
+  pop_messages (bus, 2);        /* pop remaining ready => paused messages off the bus */
   ASSERT_STATE_CHANGE_MSG (bus, pipeline, GST_STATE_READY, GST_STATE_PAUSED,
       108);
-
+  pop_async_done (bus);
+#endif
   /* PAUSED => PLAYING */
+  GST_DEBUG ("popping PAUSED -> PLAYING messages");
   ASSERT_STATE_CHANGE_MSG (bus, sink, GST_STATE_PAUSED, GST_STATE_PLAYING, 109);
   ASSERT_STATE_CHANGE_MSG (bus, identity, GST_STATE_PAUSED, GST_STATE_PLAYING,
       110);
   ASSERT_STATE_CHANGE_MSG (bus, src, GST_STATE_PAUSED, GST_STATE_PLAYING, 111);
   ASSERT_STATE_CHANGE_MSG (bus, pipeline, GST_STATE_PAUSED, GST_STATE_PLAYING,
       112);
-#else
-  pop_messages (bus, 3);        /* pop remaining ready => paused messages off the bus */
-  pop_messages (bus, 4);        /* pop paused => playing messages off the bus */
-#endif
 
   /* don't set to NULL that will set the bus flushing and kill our messages */
   ret = gst_element_set_state (pipeline, GST_STATE_READY);
@@ -656,6 +680,7 @@ GST_START_TEST (test_children_state_change_order_semi_sink)
   /* READY => PAUSED */
   /* because of pre-rolling, sink will return ASYNC on state
    * change and change state later when it has a buffer */
+  GST_DEBUG ("popping READY -> PAUSED messages");
   ASSERT_STATE_CHANGE_MSG (bus, identity, GST_STATE_READY, GST_STATE_PAUSED,
       205);
 #if 0
@@ -665,19 +690,20 @@ GST_START_TEST (test_children_state_change_order_semi_sink)
    * in which case the sink will commit its new state before the source ...  */
   ASSERT_STATE_CHANGE_MSG (bus, src, GST_STATE_READY, GST_STATE_PAUSED, 206);
   ASSERT_STATE_CHANGE_MSG (bus, sink, GST_STATE_READY, GST_STATE_PAUSED, 207);
+#else
+  pop_messages (bus, 2);        /* pop remaining ready => paused messages off the bus */
   ASSERT_STATE_CHANGE_MSG (bus, pipeline, GST_STATE_READY, GST_STATE_PAUSED,
       208);
+  pop_async_done (bus);
 
   /* PAUSED => PLAYING */
+  GST_DEBUG ("popping PAUSED -> PLAYING messages");
   ASSERT_STATE_CHANGE_MSG (bus, sink, GST_STATE_PAUSED, GST_STATE_PLAYING, 209);
   ASSERT_STATE_CHANGE_MSG (bus, identity, GST_STATE_PAUSED, GST_STATE_PLAYING,
       210);
   ASSERT_STATE_CHANGE_MSG (bus, src, GST_STATE_PAUSED, GST_STATE_PLAYING, 211);
   ASSERT_STATE_CHANGE_MSG (bus, pipeline, GST_STATE_PAUSED, GST_STATE_PLAYING,
       212);
-#else
-  pop_messages (bus, 3);        /* pop remaining ready => paused messages off the bus */
-  pop_messages (bus, 4);        /* pop paused => playing messages off the bus */
 #endif
 
   /* don't set to NULL that will set the bus flushing and kill our messages */
@@ -808,6 +834,63 @@ GST_START_TEST (test_children_state_change_order_two_sink)
 
 GST_END_TEST;
 
+GST_START_TEST (test_iterate_sorted)
+{
+  GstElement *src, *tee, *identity, *sink1, *sink2, *pipeline, *bin;
+  GstIterator *it;
+  gpointer elem;
+
+  pipeline = gst_pipeline_new (NULL);
+  fail_unless (pipeline != NULL, "Could not create pipeline");
+
+  bin = gst_bin_new (NULL);
+  fail_unless (bin != NULL, "Could not create bin");
+
+  src = gst_element_factory_make ("fakesrc", NULL);
+  fail_if (src == NULL, "Could not create fakesrc");
+
+  tee = gst_element_factory_make ("tee", NULL);
+  fail_if (tee == NULL, "Could not create tee");
+
+  sink1 = gst_element_factory_make ("fakesink", NULL);
+  fail_if (sink1 == NULL, "Could not create fakesink1");
+
+  gst_bin_add_many (GST_BIN (bin), src, tee, sink1, NULL);
+
+  fail_unless (gst_element_link (src, tee) == TRUE);
+  fail_unless (gst_element_link (tee, sink1) == TRUE);
+
+  identity = gst_element_factory_make ("identity", NULL);
+  fail_if (identity == NULL, "Could not create identity");
+
+  sink2 = gst_element_factory_make ("fakesink", NULL);
+  fail_if (sink2 == NULL, "Could not create fakesink2");
+
+  gst_bin_add_many (GST_BIN (pipeline), bin, identity, sink2, NULL);
+
+  fail_unless (gst_element_link (tee, identity) == TRUE);
+  fail_unless (gst_element_link (identity, sink2) == TRUE);
+
+  it = gst_bin_iterate_sorted (GST_BIN (pipeline));
+  fail_unless (gst_iterator_next (it, &elem) == GST_ITERATOR_OK);
+  fail_unless (elem == sink2);
+  gst_object_unref (elem);
+
+  fail_unless (gst_iterator_next (it, &elem) == GST_ITERATOR_OK);
+  fail_unless (elem == identity);
+  gst_object_unref (elem);
+
+  fail_unless (gst_iterator_next (it, &elem) == GST_ITERATOR_OK);
+  fail_unless (elem == bin);
+  gst_object_unref (elem);
+
+  gst_iterator_free (it);
+
+  gst_object_unref (pipeline);
+}
+
+GST_END_TEST;
+
 Suite *
 gst_bin_suite (void)
 {
@@ -827,6 +910,7 @@ gst_bin_suite (void)
   tcase_add_test (tc_chain, test_watch_for_state_change);
   tcase_add_test (tc_chain, test_add_linked);
   tcase_add_test (tc_chain, test_add_self);
+  tcase_add_test (tc_chain, test_iterate_sorted);
 
   return s;
 }
