@@ -119,6 +119,64 @@ GST_START_TEST (test_from_string)
 
 GST_END_TEST;
 
+
+GST_START_TEST (test_to_string)
+{
+  GstStructure *st1;
+
+  ASSERT_CRITICAL (st1 = gst_structure_new ("Foo\nwith-newline", NULL));
+  fail_unless (st1 == NULL);
+
+  /* FIXME 0.11: re-enable this */
+#if 0
+  ASSERT_CRITICAL (st1 = gst_structure_new ("Foo with whitespace", NULL));
+  fail_unless (st1 == NULL);
+#else
+  st1 = gst_structure_new ("Foo with whitespace is still allowed", NULL);
+  fail_unless (st1 != NULL);
+  gst_structure_free (st1);
+#endif
+
+  ASSERT_CRITICAL (st1 = gst_structure_new ("1st", NULL));
+  fail_unless (st1 == NULL);
+}
+
+GST_END_TEST;
+
+
+GST_START_TEST (test_to_from_string)
+{
+  GstCaps *caps1, *caps2;
+  GstStructure *st1, *st2;
+  gchar *str, *res1, *res2;
+
+  /* test escaping/unescaping */
+  st1 = gst_structure_new ("FooBar-123/0_1", "num", G_TYPE_INT, 9173,
+      "string", G_TYPE_STRING, "Something Like Face/Off", NULL);
+  str = gst_structure_to_string (st1);
+  st2 = gst_structure_from_string (str, NULL);
+  g_free (str);
+
+  fail_unless (st2 != NULL);
+
+  /* need to put stuctures into caps to compare */
+  caps1 = gst_caps_new_empty ();
+  gst_caps_append_structure (caps1, st1);
+  caps2 = gst_caps_new_empty ();
+  gst_caps_append_structure (caps2, st2);
+  res1 = gst_caps_to_string (caps1);
+  res2 = gst_caps_to_string (caps2);
+  fail_unless (gst_caps_is_equal (caps1, caps2),
+      "Structures did not match:\n\tStructure 1: %s\n\tStructure 2: %s\n",
+      res1, res2);
+  gst_caps_unref (caps1);
+  gst_caps_unref (caps2);
+  g_free (res1);
+  g_free (res2);
+}
+
+GST_END_TEST;
+
 GST_START_TEST (test_complete_structure)
 {
   GstStructure *structure;
@@ -204,6 +262,85 @@ GST_START_TEST (test_fixate)
 
 GST_END_TEST;
 
+GST_START_TEST (test_fixate_frac_list)
+{
+  GstStructure *s;
+  GValue list = { 0 };
+  GValue frac = { 0 };
+  gchar *str;
+  gint num, denom;
+
+  g_value_init (&list, GST_TYPE_LIST);
+  g_value_init (&frac, GST_TYPE_FRACTION);
+
+  gst_value_set_fraction (&frac, 30, 1);
+  gst_value_list_append_value (&list, &frac);
+  gst_value_set_fraction (&frac, 15, 1);
+  gst_value_list_append_value (&list, &frac);
+  gst_value_set_fraction (&frac, 10, 1);
+  gst_value_list_append_value (&list, &frac);
+
+  s = gst_structure_new ("name", NULL);
+  gst_structure_set_value (s, "frac", &list);
+  g_value_unset (&frac);
+  g_value_unset (&list);
+
+  str = gst_structure_to_string (s);
+  GST_DEBUG ("list %s", str);
+  g_free (str);
+
+  /* fixate to the nearest fraction, this should give 15/1 */
+  fail_unless (gst_structure_fixate_field_nearest_fraction (s, "frac", 14, 1));
+
+  fail_unless (gst_structure_get_fraction (s, "frac", &num, &denom));
+  fail_unless (num == 15);
+  fail_unless (denom == 1);
+
+  gst_structure_free (s);
+}
+
+GST_END_TEST;
+
+GST_START_TEST (test_structure_nested)
+{
+  GstStructure *sp, *sc1, *sc2;
+  gchar *str;
+
+  sc1 =
+      gst_structure_new ("Camera", "XResolution", G_TYPE_INT, 72, "YResolution",
+      G_TYPE_INT, 73, NULL);
+  fail_unless (sc1 != NULL);
+
+  sc2 =
+      gst_structure_new ("Image-Data", "Orientation", G_TYPE_STRING, "top-left",
+      NULL);
+  fail_unless (sc2 != NULL);
+
+  sp = gst_structure_new ("Exif", "Camera", GST_TYPE_STRUCTURE, sc1,
+      "Image Data", GST_TYPE_STRUCTURE, sc2, NULL);
+  fail_unless (sp != NULL);
+
+  fail_unless (gst_structure_has_field_typed (sp, "Camera",
+          GST_TYPE_STRUCTURE));
+
+  str = gst_structure_to_string (sp);
+  fail_unless (str != NULL);
+
+  fail_unless (g_str_equal (str,
+          "Exif"
+          ", Camera=(structure)Camera, XResolution=(int)72, YResolution=(int)73;"
+          ", Image Data=(structure)Image-Data, Orientation=(string)top-left;;"));
+
+  g_free (str);
+  str = NULL;
+
+  gst_structure_free (sc1);
+  gst_structure_free (sc2);
+  gst_structure_free (sp);
+
+}
+
+GST_END_TEST;
 
 Suite *
 gst_structure_suite (void)
@@ -214,9 +351,13 @@ gst_structure_suite (void)
   suite_add_tcase (s, tc_chain);
   tcase_add_test (tc_chain, test_from_string_int);
   tcase_add_test (tc_chain, test_from_string);
+  tcase_add_test (tc_chain, test_to_string);
+  tcase_add_test (tc_chain, test_to_from_string);
   tcase_add_test (tc_chain, test_complete_structure);
   tcase_add_test (tc_chain, test_structure_new);
   tcase_add_test (tc_chain, test_fixate);
+  tcase_add_test (tc_chain, test_fixate_frac_list);
+  tcase_add_test (tc_chain, test_structure_nested);
   return s;
 }
 
