@@ -33,7 +33,7 @@
 #include <locale.h>
 #include <glib/gprintf.h>
 
-static char *_name;
+static char *_name = NULL;
 
 static int print_element_info (GstElementFactory * factory,
     gboolean print_names);
@@ -246,14 +246,19 @@ print_interfaces (GType type)
 
   if (ifaces) {
     if (n_ifaces) {
-      g_print ("%s", _name);
+      if (_name)
+        g_print (_name);
       g_print (_("Implemented Interfaces:\n"));
       iface = ifaces;
       while (*iface) {
-        g_print ("%s  %s\n", _name, g_type_name (*iface));
+        if (_name)
+          g_print (_name);
+        g_print ("  %s\n", g_type_name (*iface));
         iface++;
       }
-      g_print ("%s\n", _name);
+      if (_name)
+        g_print (_name);
+      g_print ("\n");
       g_free (ifaces);
     }
   }
@@ -288,19 +293,25 @@ print_element_properties_info (GstElement * element)
     if (param->flags & G_PARAM_READABLE) {
       g_object_get_property (G_OBJECT (element), param->name, &value);
       readable = TRUE;
-      g_print ((first_flag ? "" : ", "));
+      if (!first_flag)
+        g_print (", ");
+      else
+        first_flag = FALSE;
       g_print (_("readable"));
-      first_flag = FALSE;
     }
     if (param->flags & G_PARAM_WRITABLE) {
-      g_print ((first_flag ? "" : ", "));
+      if (!first_flag)
+        g_print (", ");
+      else
+        first_flag = FALSE;
       g_print (_("writable"));
-      first_flag = FALSE;
     }
     if (param->flags & GST_PARAM_CONTROLLABLE) {
-      g_print ((first_flag ? "" : ", "));
+      if (!first_flag)
+        g_print (", ");
+      else
+        first_flag = FALSE;
       g_print (_("controllable"));
-      first_flag = FALSE;
     }
     n_print ("\n");
 
@@ -456,9 +467,11 @@ print_element_properties_info (GstElement * element)
 
           j = 0;
           while (values[j].value_name) {
-            g_print ("\n%s%-23.23s    (%d): %-16s - %s", "",
-                _name, values[j].value, values[j].value_nick,
-                values[j].value_name);
+            g_print ("\n");
+            if (_name)
+              g_print (_name);
+            g_print ("%-23.23s    (%d): %-16s - %s", "",
+                values[j].value, values[j].value_nick, values[j].value_name);
             j++;
           }
           /* g_type_class_unref (ec); */
@@ -488,9 +501,11 @@ print_element_properties_info (GstElement * element)
 
           j = 0;
           while (values[j].value_name) {
-            g_print ("\n%s%-23.23s    (0x%08x): %-16s - %s", "",
-                _name, values[j].value, values[j].value_nick,
-                values[j].value_name);
+            g_print ("\n");
+            if (_name)
+              g_print (_name);
+            g_print ("%-23.23s    (0x%08x): %-16s - %s", "",
+                values[j].value, values[j].value_nick, values[j].value_name);
             j++;
           }
 
@@ -606,8 +621,6 @@ print_implementation_info (GstElement * element)
   n_print ("\n");
   n_print ("Element Implementation:\n");
 
-  n_print ("  No loopfunc(), must be chain-based or not configured yet\n");
-
   n_print ("  Has change_state() function: %s\n",
       GST_DEBUG_FUNCPTR_NAME (gstelement_class->change_state));
 #ifndef GST_DISABLE_LOADSAVE
@@ -797,21 +810,25 @@ print_signal_info (GstElement * element)
           g_type_name (query->return_type), g_type_name (type));
 
       for (j = 0; j < query->n_params; j++) {
+        if (_name)
+          g_print (_name);
         if (G_TYPE_IS_FUNDAMENTAL (query->param_types[j])) {
-          g_print (",\n%s%s%s arg%d", _name, indent,
+          g_print (",\n%s%s arg%d", indent,
               g_type_name (query->param_types[j]), j);
         } else if (G_TYPE_IS_ENUM (query->param_types[j])) {
-          g_print (",\n%s%s%s arg%d", _name, indent,
+          g_print (",\n%s%s arg%d", indent,
               g_type_name (query->param_types[j]), j);
         } else {
-          g_print (",\n%s%s%s* arg%d", _name, indent,
+          g_print (",\n%s%s* arg%d", indent,
               g_type_name (query->param_types[j]), j);
         }
       }
 
-      if (k == 0)
-        g_print (",\n%s%sgpointer user_data);\n", _name, indent);
-      else
+      if (k == 0) {
+        if (_name)
+          g_print (_name);
+        g_print (",\n%sgpointer user_data);\n", indent);
+      } else
         g_print (");\n");
 
       g_free (indent);
@@ -876,7 +893,8 @@ print_element_list (gboolean print_all)
           print_element_info (factory, TRUE);
         else
           g_print ("%s:  %s: %s\n", plugin->desc.name,
-              GST_PLUGIN_FEATURE_NAME (factory), factory->details.longname);
+              GST_PLUGIN_FEATURE_NAME (factory),
+              gst_element_factory_get_longname (factory));
       }
 #ifndef GST_DISABLE_INDEX
       else if (GST_IS_INDEX_FACTORY (feature)) {
@@ -972,7 +990,7 @@ print_plugin_features (GstPlugin * plugin)
 
       factory = GST_ELEMENT_FACTORY (feature);
       n_print ("  %s: %s\n", GST_PLUGIN_FEATURE_NAME (factory),
-          factory->details.longname);
+          gst_element_factory_get_longname (factory));
       num_elements++;
     }
 #ifndef GST_DISABLE_INDEX
@@ -991,14 +1009,16 @@ print_plugin_features (GstPlugin * plugin)
       if (factory->extensions) {
         guint i = 0;
 
-        g_print ("%s type: ", plugin->desc.name);
+        g_print ("%s: %s: ", plugin->desc.name,
+            gst_plugin_feature_get_name (feature));
         while (factory->extensions[i]) {
           g_print ("%s%s", i > 0 ? ", " : "", factory->extensions[i]);
           i++;
         }
         g_print ("\n");
       } else
-        g_print ("%s type: N/A\n", plugin->desc.name);
+        g_print ("%s: %s: no extensions\n", plugin->desc.name,
+            gst_plugin_feature_get_name (feature));
 
       num_types++;
     } else {
@@ -1071,7 +1091,7 @@ print_element_info (GstElementFactory * factory, gboolean print_names)
   if (print_names)
     _name = g_strdup_printf ("%s: ", GST_PLUGIN_FEATURE (factory)->name);
   else
-    _name = "";
+    _name = NULL;
 
   print_factory_details_info (factory);
   if (GST_PLUGIN_FEATURE (factory)->plugin_name) {
@@ -1097,19 +1117,151 @@ print_element_info (GstElementFactory * factory, gboolean print_names)
   print_signal_info (element);
   print_children_info (element);
 
-  if (_name[0] != '\0')
-    g_free (_name);
+  gst_object_unref (factory);
+  g_free (_name);
 
   return 0;
 }
 
+
+static void
+print_plugin_automatic_install_info_codecs (GstElementFactory * factory)
+{
+  GstPadDirection direction;
+  const gchar *type_name;
+  const gchar *klass;
+  const GList *static_templates, *l;
+  GstCaps *caps = NULL;
+  guint i, num;
+
+  klass = gst_element_factory_get_klass (factory);
+  g_return_if_fail (klass != NULL);
+
+  if (strstr (klass, "Demuxer") ||
+      strstr (klass, "Decoder") ||
+      strstr (klass, "Depay") || strstr (klass, "Parser")) {
+    type_name = "decoder";
+    direction = GST_PAD_SINK;
+  } else if (strstr (klass, "Muxer") ||
+      strstr (klass, "Encoder") || strstr (klass, "Pay")) {
+    type_name = "encoder";
+    direction = GST_PAD_SRC;
+  } else {
+    return;
+  }
+
+  /* decoder/demuxer sink pads should always be static and there should only
+   * be one, the same applies to encoders/muxers and source pads */
+  static_templates = gst_element_factory_get_static_pad_templates (factory);
+  for (l = static_templates; l != NULL; l = l->next) {
+    GstStaticPadTemplate *tmpl = NULL;
+
+    tmpl = (GstStaticPadTemplate *) l->data;
+    if (tmpl->direction == direction) {
+      caps = gst_static_pad_template_get_caps (tmpl);
+      break;
+    }
+  }
+
+  if (caps == NULL) {
+    g_printerr ("Couldn't find static pad template for %s '%s'\n",
+        type_name, GST_PLUGIN_FEATURE_NAME (factory));
+    return;
+  }
+
+  caps = gst_caps_make_writable (caps);
+  num = gst_caps_get_size (caps);
+  for (i = 0; i < num; ++i) {
+    GstStructure *s;
+    gchar *s_str;
+
+    s = gst_caps_get_structure (caps, i);
+    /* remove fields that are almost always just MIN-MAX of some sort
+     * in order to make the caps look less messy */
+    gst_structure_remove_field (s, "pixel-aspect-ratio");
+    gst_structure_remove_field (s, "framerate");
+    gst_structure_remove_field (s, "channels");
+    gst_structure_remove_field (s, "width");
+    gst_structure_remove_field (s, "height");
+    gst_structure_remove_field (s, "rate");
+    gst_structure_remove_field (s, "depth");
+    gst_structure_remove_field (s, "clock-rate");
+    s_str = gst_structure_to_string (s);
+    g_print ("%s-%s\n", type_name, s_str);
+    g_free (s_str);
+  }
+  gst_caps_unref (caps);
+}
+
+static void
+print_plugin_automatic_install_info_protocols (GstElementFactory * factory)
+{
+  gchar **protocols, **p;
+
+  protocols = gst_element_factory_get_uri_protocols (factory);
+  if (protocols != NULL && *protocols != NULL) {
+    switch (gst_element_factory_get_uri_type (factory)) {
+      case GST_URI_SINK:
+        for (p = protocols; *p != NULL; ++p)
+          g_print ("urisink-%s\n", *p);
+        break;
+      case GST_URI_SRC:
+        for (p = protocols; *p != NULL; ++p)
+          g_print ("urisource-%s\n", *p);
+        break;
+      default:
+        break;
+    }
+    g_strfreev (protocols);
+  }
+}
+
+static void
+print_plugin_automatic_install_info (GstPlugin * plugin)
+{
+  const gchar *plugin_name;
+  GList *features, *l;
+
+  plugin_name = gst_plugin_get_name (plugin);
+
+  /* not interested in typefind factories, only element factories */
+  features = gst_registry_get_feature_list (gst_registry_get_default (),
+      GST_TYPE_ELEMENT_FACTORY);
+
+  for (l = features; l != NULL; l = l->next) {
+    GstPluginFeature *feature;
+
+    feature = GST_PLUGIN_FEATURE (l->data);
+
+    /* only interested in the ones that are in the plugin we just loaded */
+    if (g_str_equal (plugin_name, feature->plugin_name)) {
+      GstElementFactory *factory;
+
+      g_print ("element-%s\n", gst_plugin_feature_get_name (feature));
+
+      factory = GST_ELEMENT_FACTORY (feature);
+      print_plugin_automatic_install_info_protocols (factory);
+      print_plugin_automatic_install_info_codecs (factory);
+    }
+  }
+
+  g_list_foreach (features, (GFunc) gst_object_unref, NULL);
+  g_list_free (features);
+}
+
 int
 main (int argc, char *argv[])
 {
   gboolean print_all = FALSE;
+  gboolean print_aii = FALSE;
   GOptionEntry options[] = {
     {"print-all", 'a', 0, G_OPTION_ARG_NONE, &print_all,
         N_("Print all elements"), NULL},
+    {"print-plugin-auto-install-info", '\0', 0, G_OPTION_ARG_NONE, &print_aii,
+        N_("Print a machine-parsable list of features the specified plugin "
+              "provides.\n                                       "
+              "Useful in connection with external automatic plugin "
+              "installation mechanisms"), NULL},
     GST_TOOLS_GOPTION_VERSION,
     {NULL}
   };
@@ -1122,6 +1274,9 @@ main (int argc, char *argv[])
   textdomain (GETTEXT_PACKAGE);
 #endif
 
+  if (!g_thread_supported ())
+    g_thread_init (NULL);
+
   ctx = g_option_context_new ("[ELEMENT-NAME | PLUGIN-NAME]");
   g_option_context_add_main_entries (ctx, options, GETTEXT_PACKAGE);
   g_option_context_add_group (ctx, gst_init_get_option_group ());
@@ -1163,8 +1318,12 @@ main (int argc, char *argv[])
 
       /* if there is such a plugin, print out info */
       if (plugin) {
-        print_plugin_info (plugin);
-        print_plugin_features (plugin);
+        if (print_aii) {
+          print_plugin_automatic_install_info (plugin);
+        } else {
+          print_plugin_info (plugin);
+          print_plugin_features (plugin);
+        }
       } else {
         GError *error = NULL;
 
@@ -1172,8 +1331,12 @@ main (int argc, char *argv[])
           plugin = gst_plugin_load_file (arg, &error);
 
           if (plugin) {
-            print_plugin_info (plugin);
-            print_plugin_features (plugin);
+            if (print_aii) {
+              print_plugin_automatic_install_info (plugin);
+            } else {
+              print_plugin_info (plugin);
+              print_plugin_features (plugin);
+            }
           } else {
             g_print (_("Could not load plugin file: %s\n"), error->message);
             g_error_free (error);
